<?php

/**
 * UTF8
 * Pavel Rudol, Roman Szablatura RON Software
 *  29.11.2015
 */

namespace Ron\RonPortal;

use Exception;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\Http\Csrf;
use Ron\Http\HtmlHeaders;
use Ron\MessageManager;
use Ron\MVC\BaseViewObject;
use Ron\MVC\IcalOutputObject;
use Ron\MVC\InternalRedirect;
use Ron\MVC\JsonController;
use Ron\MVC\JsonOutputObject;
use Ron\MVC\MessageManagerViewObject;
use Ron\MVC\Model;
use Ron\DateUtils;
use Ron\MVC\OutputObject;
use Ron\MVC\Redirect;
use Ron\MVC\ViewObject;
use Ron\Personalistika\DotaznikViewObject;
use Ron\Personalistika\HodnoceniSkoleni;
use Ron\RonException;
use function Ron\ronText;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Dokumenty\DokumentOutputObject;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\RonPortalSettings;
use RonPortal\Skoleni\AbsolvovaniSkoleniForm;
use RonPortal\Skoleni\GenerovaneSkoleniRecord;
use RonPortal\Skoleni\HodnoceniUcastnikuBrowser;
use RonPortal\Skoleni\PlanovaneSkoleniBrowser;
use RonPortal\Skoleni\PlanovaneSkoleniForm;
use RonPortal\Skoleni\PozadavekNaSkoleniForm;
use RonPortal\Skoleni\PrihlaseniNaPlanovaneSkoleniController;
use RonPortal\Skoleni\PrihlaseniNaSkoleniBrowser;
use RonPortal\Skoleni\Skoleni;
use RonPortal\Skoleni\SkoleniAKurzyBrowser;
use RonPortal\Skoleni\SkoleniBrowser;
use RonPortal\Skoleni\SkoleniCalendarItem;
use RonPortal\Skoleni\SkoleniUtils;
use RonPortal\Skoleni\UcastNaSkoleniBrowser;
use RonPortal\System\AgentManager;
use RonPortal\User;


//require_once('adsapp/dokumentyutils.php');
//require_once('adsapp/b_dokumenty.php');
//require_once('person/b_skoleni.php');
//require_once('person/b_skoleniprihlaseni.php');


///**
// * Class NovyDokumentKurzuForm
// * @package Ron\RonPortal
// */
//class NovyDokumentKurzuForm extends DokumentUploadForm
//{
//
//    function __construct($database, $kodKurzu)
//    {
//        parent::__construct($database, '?menuid=skoleni&action=dokumentupload');
//
//        //Typ dokumentu false zobrazi combo v dialogu pro vyber typu dokumentu
//        //$this->typDokumentu = true;
//
//        $this->hiddenFields = [
//            'KodKurzu' => $kodKurzu,
//        ];
//    }
//}

///**
// * Class DokumentyKurzuBrowser
// * @package Ron\RonPortal
// */
//class DokumentyKurzuBrowser extends DokumentyBrowser
//{
//
//    /**
//     * DokumentyKurzuBrowser constructor.
//     * @param $database
//     * @param $kodKurzu
//     */
//    function __construct($database, $kodKurzu)
//    {
//        parent::__construct($database, "?menuid=skoleni&action=dokumentdownload&kodKurzu=$kodKurzu", 0, "");
//
//        $this->sqlFromPart =
//            'Dokument, TypDokumentu, DokumentSkoleni';
//
//        $params = $this->database->createParameters();
//
//        $this->sqlWherePart = new SqlStatement(
//            'Dokument.Id = DokumentSkoleni.Id ' .
//            ' AND TypDokumentu.Kod = Dokument.Typ ' .
//            " AND DokumentSkoleni.CisloSkoleni = " . $params ->addParameter($kodKurzu), $params);
//    }
//}




class SkoleniController extends JsonController
{   
    const XML_EVENT_BEFORE = 'SKOLENI_PRIHLASENI_BEFORE';
    const XML_EVENT_AFTER = 'SKOLENI_PRIHLASENI_AFTER';
    const XML_CONTEXT_LOGIN = 'login';
    const XML_CONTEXT_LOGOUT = 'logout';
    const NASTAVENI_PONECHAT_NAVRH = 'skoleni_ponechat_navrh';
    
    /* @var $database Database */
    private $database;


    public function __construct($database)
    {
        $this->database = $database;

        TextManager::getInstance() ->loadStrings('person.ini');
        TextManager::getInstance() ->loadStrings('dotaznik.ini');

        HtmlHeaders::getInstance() ->setIcon(RonPortalSettings::getValue('shortcut_icon') ?: 'images/icon_person.png');

        $this ->addTranslationsScript('skoleni');
    }


    /**
     *  Spusti personalniho agenta pro doplneni udaju pracovniho pomeru OsobniCislo, DatumNastupu, Kalendar, Smena do tabulky GenerovaneSkoleni.
     */
    private function doplnitPracovniPomer($seznamRC)
    {
        //pokud je predano vice RC, vytvori retezec oddeleny stredniky
        $seznamRCStr = is_array($seznamRC)
            ? implode(';', $seznamRC)
            : $seznamRC;

        if ($seznamRCStr != '') {
            try {
                AgentManager::executePersonAgent('4 -DoplnitUdajePP -RCZamestnance:"' . $seznamRCStr . '"');
            } catch (Exception $exception) {
                MessageManager::addErrorMessage($exception);
            }
        }
    }


    /**
     * Zjisti, jestli ma uzivatel pravo pro zobrazeni dokumentu skoleni
     * @param string $skoleni kod skoleni
     * @param int $dokument id dokumentu
     * @return bool
     */
    private function jePravoNaDokumentSkoleni($skoleni, $dokument)
    {
        // bud je uzivatel prihlasen na skoleni, nebo musi byt dokument verejny
        if (!(SkoleniUtils::jeOsobaNaSkoleni($this ->database, $skoleni, User::getRC()) || SkoleniUtils::jeDokumentSkoleniVerejny($this ->database, $dokument))) {
            return false;
        }

        // kontrola, jestli je dokument pripojen ke skoleni a ma se zobrazit na intrawebu
        $params = $this->database->createParameters();
        $sql =
            'SELECT Dokument.Id FROM DokumentSkoleni' .
            '  JOIN Dokument ON (Dokument.Id = DokumentSkoleni.Id)'.
            '  JOIN TypDokumentu ON (TypDokumentu.Kod = Dokument.Typ)' .
            ' WHERE CisloSkoleni = ' . $params->addParameter($skoleni) .
            '  AND Dokument.Id = ' . $params->addParameter($dokument) .
            '  AND (Dokument.ZobrazitNaIntrawebu = 1 OR (Dokument.ZobrazitNaIntrawebu = 2 AND TypDokumentu.ZobrazitNaIntrawebu = 1))';

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        return (bool)$row;
    }

    /**
     * @param string $osobaRC
     * @param string $druhSkoleni
     * @param int $datumPovinnosti
     * @param int $dokument
     * @throws Exception
     * @throws DBException
     */
    private function zkontrolujPravoNaDokumentSkoleniOsoby($osobaRC, $druhSkoleni, $datumPovinnosti, $dokument)
    {
        $pravaOsobniUdaje = Prava::vratPrava('prohl_osobni');
        $pravaSkoleni = Prava::vratPrava('skoleni_prohlizeni');

        if (!($pravaOsobniUdaje->lzeZobrazitRC($osobaRC, $datumPovinnosti, $datumPovinnosti) || $pravaSkoleni->lzeZobrazitRC($osobaRC, $datumPovinnosti, $datumPovinnosti))) {
            throw new NeniPravoException();
        }

        $params = $this->database->createParameters();

        $sql =
            "SELECT Dokument.id FROM DokumentyOsoby JOIN Dokument ON (DokumentyOsoby.Id = Dokument.Id) " .
            " JOIN TypDokumentu ON (TypDokumentu.Kod = Dokument.Typ) " .
            "WHERE DokumentyOsoby.RC = " . $params->addParameter($osobaRC) .  //RC osoby skoleni
            "  AND DokumentyOsoby.GSk_RC = " . $params->addParameter($osobaRC) .
            "  AND DokumentyOsoby.GSk_DruhSkoleni = " . $params->addParameter($druhSkoleni) .  //druh skoleni
            "  AND DokumentyOsoby.GSk_Datum = " . $params->addTimestampParameter($datumPovinnosti) .  //datum vzniku povinnosti
            "  AND (Dokument.ZobrazitNaIntrawebu = 1 OR (Dokument.ZobrazitNaIntrawebu = 2 AND TypDokumentu.ZobrazitNaIntrawebu = 1))" .
            "  AND Dokument.Id = " . $params->addParameter($dokument) ;

        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        if (!$row) {
            throw new NeniPravoException();
        }
    }

    /**
     * 
     * @return DokumentOutputObject
     * @throws Exception
     */
    private function downloadDokumentSkoleni()
    {
        $dokument = (int)$_GET['dokument'];

        // cislo skoleni
        $skoleni = $_GET['cisloskoleni'] ?? '';

        //v url neni zadano konkretni skoleni, ani osoba ktera absolvovala skoleni
        if ($skoleni === '' && !(isset($_GET['osoba']) && isset($_GET['druhskoleni']) && isset($_GET['datum']))) {
            throw new NeniPravoException();
        }

        if ($skoleni) {
            // stahuje se dokument skoleni => overi se pravo na skoleni
            if (!$this->jePravoNaDokumentSkoleni($skoleni, $dokument)) {
                throw new NeniPravoException();
            }
        } else {
            if ( !(isset($_GET['osoba']) && isset($_GET['druhskoleni']) && isset($_GET['datum'])) ) {
                throw new NeniPravoException();
            }

            // muze se stahovat i dokument osoby, pak se overuje, ze je dokument pripojen ke skoleni osoby
            $params = $this ->database ->createParameters();
            $osobaRC = DBUtils::getSqlValue($this->database,
                'SELECT RC FROM PracovniPomer' .
                ' WHERE ID = ' . $params ->addParameter((int)$_GET['osoba']),
                $params);

            $this->zkontrolujPravoNaDokumentSkoleniOsoby($osobaRC, $_GET['druhskoleni'], (int)$_GET['datum'], $dokument);
        }

        return new DokumentOutputObject($this->database,$dokument);
    }


    /**
     * @return IcalOutputObject
     * @throws Exception
     */
    private function downloadIcal()
    {
        $skoleni = isset($_GET['cisloskoleni']) ? $_GET['cisloskoleni'] : '';

        $cal = SkoleniCalendarItem::loadKurz($this->database, $skoleni);
        if (!$cal) {
            throw new Exception('Kurz nebyl nalezen!');
        }

        return new IcalOutputObject($cal);
    }


    /**
     * zobrazi seznam skoleni, na ktere je mozne se prihlasit
     * @return string
     * @throws DBException
     * @throws Exception
     */
    private function zobrazPrihlaseniSkoleni()
    {
        if (Prava::vratHodnotuPrava('skoleni_prihlasit') == 0) {
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        HtmlHeaders::getInstance()->addScript('script/personscripts.js');   // dotazy pri kliknuti na prihlasit/odhlasit

        $b = new PrihlaseniNaSkoleniBrowser($this ->database);
        $b ->load();
        return $b;
    }

    /**
     * Prihlasi zadanou osobu ($osobaIDPP) na skoleni, pokud je dostupna kapacita, uzivatel ma na prihlaseni pravo
     * Vraci RC prihlasene osoby
     * @param string $cisloSkoleni kod skoleni
     * @param string $osobaIDPP osoba, ktera se prihlasuje na skoleni
     * @param int $datumPovinnosti
     * @param bool $spustitAgenta
     * @return string
     * @throws DBException
     * @throws Exception
     */
    private function prihlaseniSkoleni($cisloSkoleni, $osobaIDPP, $datumPovinnosti, $spustitAgenta = true)
    {
        $skoleni = Skoleni::nactiSkoleni($this ->database, $cisloSkoleni);

        $pp = PracovniPomer::nactiPP($this ->database, $osobaIDPP);

        $prava = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        if (!$prava->lzeZobrazitRC($pp->rc)) {
            throw new NeniPravoException();
        }

        //skoleni musi byt ve stavu pripravuje se = 0 nebo probiha = 1
        if ($skoleni ->stavSkoleni != Skoleni::STAV_PRIPRAVUJE_SE  &&  $skoleni ->stavSkoleni != Skoleni::STAV_PROBIHA) {
            throw new Exception(sprintf(ronText('SkoleniChybnyStav'), $skoleni ->popis));
        }

        if ($skoleni ->cisloPlanovanehoSkoleni) {
            if (!SkoleniUtils::jeOsobaNaPlanovanemSkoleni($this ->database, $skoleni ->cisloPlanovanehoSkoleni, $pp->rc)) {
                throw new Exception(sprintf(ronText('SkoleniNeniPrihlasenPlan'), $pp->jmenoPrijmeniKratke, $skoleni ->cisloPlanovanehoSkoleni));
            }
        }

        //kontrola kapacity a jednoho prihlaseni osoby se provadi pro bezne kurzy, u e-learningovych se ignoruje
        if (!$skoleni ->elearning) {
            if (!$skoleni->jeVolnaKapacitaSkoleni($this ->database)) {
                throw new Exception(sprintf(ronText('SkoleniNaplnenaKapacita'), $skoleni ->popis, $pp->jmenoPrijmeniKratke));
            }

            if (!SkoleniUtils::jeVolnaKapacitaMistnosti($this ->database, $cisloSkoleni, $pp->rc)) {
                throw new Exception(sprintf(ronText('SkoleniNaplnenaKapacitaMistnosti'), $skoleni ->popis, $pp->jmenoPrijmeniKratke));
            }

            if (SkoleniUtils::jeOsobaNaSkoleni($this->database, $cisloSkoleni, $pp->rc)) {
                throw new Exception(sprintf(ronText('SkoleniJizPrihlasene'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
            }
        } else {
            if (SkoleniUtils::pocetNeabsolvovanychEKurzu($this ->database, $cisloSkoleni, $pp->rc) > 0) {
                throw new Exception(sprintf(ronText('SkoleniExistujiNeabsolvaneEkurzy'), $pp->jmenoPrijmeniKratke));
            }
        }

        //pro pouziti v XML k identifikaci vybraneho PP, napr. pro zapis do PK
        $xmlVariables = [
            'osobaRC' => $pp ->rc,
            'osobaOsobniCislo' => $pp ->osobniCislo,
            'osobaDatumNastupu' => $pp ->datumNastupu
        ];

        $uspesnaRegistrace = false;

        // TODO: predelat na GenerovaneSkoleniRec
        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni', $this);

        $record->addStringField('RC', true, 15, $pp->rc);
        $record->addLogText('  ' . $pp ->jmenoPrijmeni);
        $record->addStringField('DruhSkoleni', true, 10, $skoleni ->druhSkoleni);
        $record->addTimestampField('DatumVznikuPovinnosti', true, DateUtils::now());

        $record->addDateField('DatumSkoleni', false, $skoleni ->datumPocatkuSkoleni);
        $record->addStringField('CisloSkoleni', false, 40, $skoleni ->cisloSkoleni);
        $record->addDateField('DatumPrihlasky', false, time());
        $record->addIntField('PrihlasenoZAplikace', false, 1);
        $record->addStringField('Prihlasil', false, 25, User::getLoginName());

        $record->addIntField('Potvrzeni', true);

        $record->addIntField('SchvaleniPrihlasky', true);

        // dalsi readonly pole
        $record->addIntField('DuvodSkoleni', true, 1); //individualni duvod skoleni
        $record->addDateField('DatumZapisu', true, DateUtils::today());
        $record->addStringField('Vlozil', true, 25, User::getLoginName());
        $record->addDateField('PozadovaneDatumSkoleni', true, $skoleni ->datumPocatkuSkoleni);


        if ($skoleni ->pozadovatSchvaleniPrihl == 1) {
            //pokud ma uzivatel pravo na potvrzeni prihlaseni, potvzeni zapise, jinak se provadi samostatnou akci
            if ($pravaPotvrditPrihlaseni->lzeZobrazitRC($pp->rc)) {
                $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(1);
                $record->addStringField('Vyhodnotil', false, 25, User::getLoginName());
                $record->addDateField('DatumVyhodnoceni', false, time());
                $record->addMemoField('PoznamkaSchvalovani', 'memo', false);
            } else {
                $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(0);
                $uspesnaRegistrace = true;
            }
        } else {
            $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(-1);
        }

        $record->addIntField('Generovane', true, 0);

        $params = $this->database->createParameters();

        //prihlasovani na jiz vygenerovany/rucne vlozeny zaznam
        if ($datumPovinnosti > 0) {
            $record->setWhere(
                "RC = " . $params->addParameter($pp->rc) .
                " AND DruhSkoleni = " . $params->addParameter($skoleni ->druhSkoleni) .
                " AND DatumVznikuPovinnosti = " . $params->addTimestampParameter($datumPovinnosti),
                $params);
        } else {
            $record->setWhere(
                "RC = " . $params->addParameter($pp->rc) .
                " AND DruhSkoleni = " . $params->addParameter($skoleni ->druhSkoleni) .
                " AND DatumSkoleni IS NULL " .
                " AND NOT EXISTS(SELECT * FROM GenerovaneSkoleni GS WHERE GS.RC = GenerovaneSkoleni.RC " .
                "  AND GS.DruhSkoleni = GenerovaneSkoleni.DruhSkoleni " .
                "  AND GS.DatumVznikuPovinnosti > GenerovaneSkoleni.DatumVznikuPovinnosti " .
                "  AND GS.CisloSkoleni IS NOT NULL)", $params);
        }

        if ($record->loadRecord()) {
            //osoba jiz ma vygenerovanou povinnost skoleni absolvovat, zapisou se informace o prihlasenem skoleni
            //pro povinnost = 0 (doporucene) se pri schvaleni nastavi potvrzeni na 1
            if ($record->findFieldByName('Potvrzeni')->value == 0)
                if (($record->findFieldByName('SchvaleniPrihlasky')->value == -1) || ($record->findFieldByName('SchvaleniPrihlasky')->value == 1)) {
                    $record->findFieldByName('Potvrzeni')->setNewValue(1);
                }

            if (!$record->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_LOGIN, $xmlVariables)) {
                throw new Exception(ronText('SkoleniChybaXMLudalosti'));
            }
            
            if (!$record->updateRecord()) {
                throw new Exception(sprintf(ronText('SkoleniChybaPrihlaseni'), $skoleni ->popis, $pp->jmenoPrijmeniKratke));
            }
        } else {
            //pro osobu neexistuje povinnost absolvovat skoleni (skoleni je doporucene) vlozi se novy zaznam
            $record->findFieldByName('Potvrzeni')->setNewValue(0);

            //pro schvalene nebo pokud se nevyzaduje schvaleni nastavi se potvrzeni na 1
            if (($record['SchvaleniPrihlasky'] == -1) || ($record['SchvaleniPrihlasky'] == 1)) {
                $record->findFieldByName('Potvrzeni')->setNewValue(1);
            }

            if (!$record->executeXmlEvents(self::XML_EVENT_BEFORE, self::XML_CONTEXT_LOGIN, $xmlVariables)) {
                throw new Exception(ronText('SkoleniChybaXMLudalosti'));
            }

            $record->insertRecord();
        }
        
        if (!$record->executeXmlEvents(self::XML_EVENT_AFTER, self::XML_CONTEXT_LOGIN, $xmlVariables)) {
            throw new Exception(ronText('SkoleniChybaXMLudalosti'));
        }

        if ($uspesnaRegistrace) {
            MessageManager::addInfoMessage(sprintf(ronText('Skoleni_uspesna_registrace'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
        } else {
            MessageManager::addInfoMessage(sprintf(ronText('SkoleniUspesnePrihlaseni'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
        }

        if ($spustitAgenta) {
            $this->doplnitPracovniPomer($pp->rc);
        }

        return $pp ->rc;
    }


    /**
     * Provede odhlaseni osoby ze skoleni (smaze zaznam, nebo zrusi prirazeni skoleni)
     * @param string $cisloSkoleni kod skoleni
     * @param string $osobaIDPP rc osoby
     * @param int $datumPovinnosti
     * @throws Exception
     */
    private function odhlaseniSkoleni($cisloSkoleni, $osobaIDPP, $datumPovinnosti)
    {
        $skoleni = Skoleni::nactiSkoleni($this ->database, $cisloSkoleni);

        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osobaIDPP);

        //$json = isset($_GET['json']);

        $prava = Prava::vratPrava('skoleni_prihlasit');
        $pravaPotvrditPrihlaseni = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        if (!$prava->lzeZobrazitRC($pp->rc)) {
            throw new NeniPravoException();
        }

        $smazat = isset($_GET['navrh']) && ($_GET['navrh'] == 'smazat');
        //pokud je v nastaveni zvoleno ponechat navrh, nikdy se nebude mazat
        if (RonPortalSettings::getIntValue(self::NASTAVENI_PONECHAT_NAVRH)) {
            $smazat = false;
        }

        //if (!$this->jeOsobaNaSkoleni($cisloSkoleni, $pp->rc)) {
        if (!SkoleniUtils::jeOsobaNaSkoleni($this ->database, $cisloSkoleni, $pp->rc)) {
            throw new Exception(sprintf(ronText('SkoleniNeniPrihlasene'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
        }

        $xmlVariables = [
            'osobaRC' => $pp->rc,
            'osobaOsobniCislo' => $pp->osobniCislo,
            'osobaDatumNastupu' => $pp->datumNastupu
        ];

//        if (!RonPortalSession::vsechnyOsoby()) {
//            //pro pouziti v XML k identifikace vybraneho PP, napr. pro zapis do PK
//            $xmlVariables = ['osobaRC' => RonPortalSession::osobaRC(),
//                'osobaOsobniCislo' => RonPortalSession::osobaOsobniCislo(),
//                'osobaDatumNastupu' => RonPortalSession::osobaNastup()];
//        }

        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni', $this);

        $record->addStringField('RC', true);
        $record->addLogText('  ' . $pp ->jmenoPrijmeni);
        $record->addStringField('DruhSkoleni', true, 10);
        $record->addTimestampField('DatumVznikuPovinnosti', true);

        $record->addStringField('CisloSkoleni', false, 40, null);
        $record->addDateField('DatumSkoleni', false, null);
        $record->addDateField('DatumPrihlasky', false, null);
        $record->addStringField('Prihlasil', false, 25, null);

        $record->addIntField('DuvodSkoleni', true);
        $record->addIntField('Potvrzeni', true);
        $record->addIntField('SchvaleniPrihlasky', true);

        $params = $this->database->createParameters();

        $record->setWhere(
            "RC = " . $params->addParameter($pp->rc) .
            " AND CisloSkoleni = " . $params->addParameter($cisloSkoleni) .
            ($datumPovinnosti > 0
                ? " AND DatumVznikuPovinnosti = " . $params->addTimestampParameter($datumPovinnosti)
                : ''),
            $params);
        
        if (!$record->loadRecord()) {
            throw new Exception(ronText('SkoleniChybaNacteniZaznamu'));
        }
        
        if (!$record->executeXmlEvents(self::XML_EVENT_BEFORE, $smazat ? 'delete' : self::XML_CONTEXT_LOGOUT, $xmlVariables)) {
            throw new Exception(ronText('SkoleniChybaXMLudalosti'));
        }

        if (!$smazat) {
            //skoleni se nemaze, jen se zrusi prirazene skoleni
            //prihlaska jiz byla schvalena, odhlaseni muze provest jen uzivatel s pravy pro schvaleni prihlasky, pri odhlaseni se zrusi zaroven schvaleni prihlasky
            if ($record->findFieldByName('SchvaleniPrihlasky')->value == 1) {
                if (!$pravaPotvrditPrihlaseni->lzeZobrazitRC($pp->rc))
                    throw new NeniPravoException();

                    $record->findFieldByName('SchvaleniPrihlasky')->setNewValue(0);

                //pro doporucene (individualni duvod skoleni - teoreticky i rucne vlozeny zaznam) se nastavi potvrzeni na 0
                if ($record->findFieldByName('DuvodSkoleni')->value == 1)
                    $record->findFieldByName('Potvrzeni')->setNewValue(0);
            }

            $record->updateRecord();
        } else {
            //smaze se zaznam generovaneho skoleni
            $record->doDelete('');
        }
        
        if (!$record->executeXmlEvents(self::XML_EVENT_AFTER, $smazat ? 'delete' : self::XML_CONTEXT_LOGOUT, $xmlVariables)) {
            throw new Exception(ronText('SkoleniChybaXMLudalosti'));
        }

//        if ($json) {
//            return new JsonOutputObject(['text' => StringUtils::jsonText(sprintf(ronText('SkoleniUspesneOdhlaseni'), $pp->jmenoPrijmeniKratke, $skoleni ->popis))]);
//        }

        MessageManager::addInfoMessage(sprintf(ronText('SkoleniUspesneOdhlaseni'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
    }


    /**
     * provede potvrzeni(zamitnuti) prihlaseni osoby na skoleni (nastavi schvaleniprihlasky na hodnotu)
     * @param string $cisloSkoleni kod skoleni
     * @param string $osobaIDPP rc osoby
     * @param int $datumPovinnosti
     * @param int $hodnota - hodnota schvaleni: 1 = schvaleno, 2 = vraceno
     * @throws Exception
     */
    private function nastavSchvaleniPrihlaseni($cisloSkoleni, $osobaIDPP, $datumPovinnosti, $hodnota = 1)
    {
        Csrf::validateCSRFGetParam();

        $pp = new PracovniPomer();
        $pp->nactiPPPodleId($this->database, $osobaIDPP);

        $skoleni = Skoleni::nactiSkoleni($this ->database, $cisloSkoleni);

        if (!SkoleniUtils::jeOsobaNaSkoleni($this ->database, $cisloSkoleni, $pp->rc)) {
            throw new Exception(sprintf(ronText('SkoleniNeniPrihlasene'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
        }

        $prava = Prava::vratPrava('skoleni_schvalit_prihlaseni');

        if (!$prava->lzeZobrazitRC($pp->rc)) {
            throw new NeniPravoException();
        }

        $record = new GenerovaneSkoleniRecord($this ->database, $this);

        $params = $this->database->createParameters();
        $record->setWhere(
            'RC = ' . $params->addParameter($pp->rc) .
            ' AND CisloSkoleni = ' . $params->addParameter($cisloSkoleni) . // POZOR! TOTO NENI PRIMARNI KLIC! Predelat?
            ' AND DatumVznikuPovinnosti = ' . $params->addTimestampParameter($datumPovinnosti),
            $params);

        if (!$record->loadRecord()) {
            throw new Exception(ronText('SkoleniChybaNacteniZaznamu'));
        }

        $record->fVyhodnotil->setNewValue(User::getLoginName());
        $record->fDatumVyhodnoceni->setNewValue(DateUtils::today());
        $record->fSchvaleniPrihlasky->setNewValue($hodnota);

        //pro doporucene nastavi pri schvaleni potvrzeni na 1, pri zamitnuti necha 0
        if ((!$record->fPotvrzeni->value)) {
            if ($hodnota == 1) {
                $record->fPotvrzeni->setNewValue(1);
                MessageManager::addInfoMessage(sprintf(ronText('Skoleni_uspesne_schvaleni'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
            } else {
                $record->fPotvrzeni->setNewValue(0);
                MessageManager::addInfoMessage(sprintf(ronText('Skoleni_uspesne_zamitnuti'), $pp->jmenoPrijmeniKratke, $skoleni ->popis));
            }
        } elseif ($hodnota == 2) { //zamitnuti schvaleni zrusi u povinneho skoleni vazbu na kurz (cislo skoleni)
            $record->fCisloSkoleni->setNewValue(null);
            $record->fDatumSkoleni->setNewValue(null);
            $record->fDatumPrihlasky->setNewValue(null);
            $record->fPrihlasil->setNewValue(null);
        }

        $record->updateRecord();
    }

    /**
     * Zobrazi prehled skoleni s filtrem na osobu
     * @return string
     * @throws DBException
     * @throws Exception
     */
    private function prehledSkoleni()
    {
        $this->hlavickaVyberAkce();

        $browser = new SkoleniBrowser($this->database);
        $browser->load();

        return $browser;
    }

    /** Zobrazi/zpracuje formular absolvovani skoleni.
     * @return Redirect|AbsolvovaniSkoleniForm
     * @throws DBException
     * @throws Exception
     */
    private function absolvovaniSkoleniFormular()
    {
        $this->hlavickaVyberAkce();

        $osoba = isset($_POST['osoba'])
            ? (int)$_POST['osoba']
            : (isset($_GET['osoba']) ? (int)$_GET['osoba'] : null);

        $druhSkoleni = isset($_POST['DruhSkoleni'])
            ? $_POST['DruhSkoleni']
            : (isset($_GET['druhskoleni']) ? $_GET['druhskoleni'] : null);

        $datumPovinnosti = isset($_POST['DatumPovinnosti'])
            ? (int)$_POST['DatumPovinnosti']
            : (isset($_GET['datumpovinnosti']) ? (int)$_GET['datumpovinnosti'] : null);

        $pp = $osoba
            ? PracovniPomer::nactiPP($this->database, $osoba)
            : null;

//        //Prava::nastavObdobiProPrava(isset($datumPovinnosti) ? $datumPovinnosti : DateUtils::now());
//        $prava = Prava::vratPrava('skoleni_absolvovani');
//
//        if ($formaction === 'new') {
//            //pri zobrazeni formulare pro vlozeni musi mit nejake prava
//            if (!$prava->jsouNejakePrava()) {
//                throw new NeniPravoException();
//            }
//        }
//        else {
//            //pri vkladani vybrane osoby, editace zaznamu s osobou musi mit pravo na konkretni osobu
//            if (!$prava->lzeZobrazitRC($pp->rc, DateUtils::now(), DateUtils::now())) {
//                throw new NeniPravoException();
//            }
//        }

        $form = new AbsolvovaniSkoleniForm($this->database, $pp, $druhSkoleni, $datumPovinnosti);

        if ($form->handleAction()) {
            return new Redirect(null, '?menuid=skoleni');
        }
        
        return $form;
    }

    /**
     * Menu skolení
     */
    private function hlavickaVyberAkce()
    {
        $sekce = [];

        if (Prava::vratHodnotuPrava('skoleni_prohlizeni')) {
            $sekce['prehled'] = ronText('Skoleni_prehled');
        }

        if (Prava::vratHodnotuPrava('skoleni_vyslat')) {
            $sekce['vyslatnaskoleni'] = ronText('Skoleni_vyslat_na_skoleni');
        }

        if (Prava::vratHodnotuPrava('skoleni_absolvovani')) {
            $sekce['zapisabsolvovani'] = ronText('Skoleni_zapsat_absolvovani');
        }

        if (Prava::vratHodnotuPrava('skoleni_prihlasit')) {
            $sekce['zobrazprihlaseni'] = ronText('MI_Prihlaseni_na_skoleni');
        }

        if (Prava::vratHodnotuPrava('skoleni_prihlasit')) {
            $sekce['skolenikurzy'] = ronText('MI_Skoleni_a_kurzy');
            $sekce['prihlaseninaplanovaneskoleni'] = ronText('MI_prihlasovani_na_planovane_skoleni');
        }

        if (Prava::vratHodnotuPrava('skoleni_plan') != 0) {
            $sekce['planovaneskolenikurzy'] = ronText('MI_Planovane_Skoleni_Kurzy');
        }

//        if (Prava::vratHodnotuPrava('skoleni_plan') != 0) {
//            $sekce['vytvoreniskolenikurzu'] = ronText('MI_Zadat_novy_plan_kurzu');
//        }

        $aktivni_sekce = ((isset($_GET['action']) && $_GET['action'])
            ? $_GET['action']
            : (isset($_GET['formaction']) ? 'zapisabsolvovani' : ''));

        Model::addData('header',  ronText('Skoleni'));
        Model::addData('aktivni_sekce', $aktivni_sekce);
        Model::addData('sekce', $sekce);
        Model::addData('url', '?menuid=skoleni&action=');
        Model::addData('wrap_class', 'skoleni');
        Model::addViewObject(new BaseViewObject('view/appsections.php'));
    }

    /** Zobrazi/zpracuje hodnoceni skoleni.
     * @return Redirect|ViewObject
     * @throws Exception
     */
    private function hodnoceniSkoleni()
    {
        $id = (int)$_GET['record'];

        if (isset($_POST['hodnoceniaction'])  &&  $_POST['hodnoceniaction']) {
            // odeslani formulare - ulozeni hodnoceni nebo uzavreni
            switch ($_POST['hodnoceniaction']) {
                case 'ulozdotaznik' :
                    return $this ->ulozHodnoceni();

                case'uzavridotaznik' :
                    return $this ->uzavriHodnoceni($id);

                default :
                    throw new Exception('Chyba v metode SkoleniController::handleAction - neznamy typ akce!');
            }
        }

        $hodnoceniAction = isset($_GET['hodnoceniaction']) ? $_GET['hodnoceniaction'] : '';
        return $this ->pripravenyDotaznik($id, $hodnoceniAction == 'editujhodnoceni', false);
    }

    /**
     * Zkontroluje, jestli ma uzivatel pravo na tento dotaznik
     * - jestli je pro zamestnance prihlaseneho na skoleni
     * - jestli je pro jeho vedouciho RCHodnoticiOsoby nebo je uzivatel na primo nadrizenem rozdeleni
     * - vraci 0 nebo false - nema pravo,  1 - muze prohlizet,  2 - muze editovat
     * @param Database $database
     * @param int  $id
     * @param bool $editace  Pokud je $editace = true a zjisti, ze nemuze editovat, tak rovnou vraci 0 a nekontroluje, jestli lze i cist
     * @return int
     * @throws Exception
     */
    public static function vratPravoNaHodnoceni($database, $id, $editace)
    {
        // zjisti stav, uroven predavani a rozdeleni osoby
        $sql =
            'SELECT PripraveneTemaHodnoceni.Stav, PripraveneTemaHodnoceni.HodnotilRC, ' . // 0, 1
            '    PripraveneTemaHodnoceni.HodnocenyRC, HodnoceniSkoleni.TypGenerovani, ' . // 2, 3
            '    HodnoceniSkoleni.RCHodnoticiOsoby, HodnoceniSkoleni.TypRozdeleni ' . // 4, 5
            ' FROM PripraveneTemaHodnoceni JOIN HodnoceniSkoleni ON ((PripraveneTemaHodnoceni.DruhHodnoceni = HodnoceniSkoleni.DruhHodnoceni) ' .
            '           AND (PripraveneTemaHodnoceni.TemaCislo = HodnoceniSkoleni.TemaCislo))' .
            ' WHERE PripraveneTemaHodnoceni.Id = ' . (int)$id ;

        MessageManager::addDebugLog($sql);

        $cursor = $database ->getSqlResult($sql);

        $row = $cursor ->fetch();
        if (!$row) {
            throw new Exception(StringUtils::fromUTF8("Chyba - zadaný dotazník (číslo $id) nebyl nalezen nebo není určen pro hodnocení školení!"));
        }

        $result = 2; // lze editovat

        // pokud se ma editovat, zkontroluje, jestli neni uzavren
        if ($row[0] == 2 || $row[0] == 3) {
            // dotaznik je uzavren
            if ($editace) {
                return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - dotazník nelze editovat, protože je již uzavřen!'));
            }

            $result = 1; // nelze editovat - jeste bude kontrolovat jesli lze cist
        }

        if ($row[1] == User::getRC()) return $result;
        // dotaznik vyplnil aktualni uzivatel, takze ma na jeho zobrazeni pravo
        // uz neni potreba kontrolovat stromovou strukturu

        switch ($row[3]) {
            case 1 : //dle typu rozdeleni
                $kodtypurozdeleni = $row[5];
                $obdobiod = DateUtils::today();
                $obdobido = DateUtils::today();

                $params = $database->createParameters();

                $sql =
                    'SELECT * ' . // kontrola, ze jsem (byl) v hodnocenem obdobi v primo nadrazenem rozdeleni
                    '  FROM RozdeleniOsob R_JA JOIN Rozdeleni R_Nad ON (R_Nad.RozdeleniRodice = R_Ja.KodTypuRozdeleni AND R_Nad.Rodic = R_Ja.Rozdeleni) ' .
                    '       JOIN RozdeleniOsob R_Hodnoceny ON (R_Hodnoceny.KodTypuRozdeleni = R_Nad.KodTypuRozdeleni AND R_Hodnoceny.Rozdeleni = R_Nad.Kod) ' .
                    "  WHERE R_Ja.RC = " . $params->addParameter(User::getRC()) .
                    '    AND R_JA.KodTypuRozdeleni = ' . $params->addParameter($kodtypurozdeleni) .
                    "    AND R_Hodnoceny.RC = " . $params->addParameter($row[2]) .
                    '    AND R_Ja.PlatnostOd <= ' . $params->addParameter($database->timestampToParam($obdobido)) .
                    '    AND (R_Ja.PlatnostDo >= ' . $params->addParameter($database->timestampToParam($obdobiod)) . '  OR  R_Ja.PlatnostDo IS NULL)' .
                    '    AND R_Hodnoceny.PlatnostOd <= ' . $params->addParameter($database->timestampToParam($obdobido)) .
                    '    AND (R_Hodnoceny.PlatnostDo >= ' . $params->addParameter($database->timestampToParam($obdobiod)) . '  OR  R_Hodnoceny.PlatnostDo IS NULL)';

                $cursor = $database->getSqlResult($sql, $params);

                $row = $cursor->fetch();
                if (!$row) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zobrazení zvoleného dotazníku nemáte právo! Hodnocená osoba patří do jiného rozdělení.'));
                }
                break;
            case 2 : //dle osoby
                if ($row[4] != User::getRC()) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zvolené hodnocení nemáte právo, nejste osobou určenou k hodnocení!'));
                }
                break;
            case 3 : //generovat - skoleni je vygenerovano pro ucastnika
                if ($row[2] != User::getRC()) {
                    return MessageManager::addErrorMessage(StringUtils::fromUTF8('Chyba - na zvolené hodnocení nemáte právo!'));
                }
                break;
        }

        // vse je OK
        return $result; // nastaveno drive na 1 (jen cist) nebo 2 (i editace)
    }

    /**
     * @param int  $id
     * @param bool $editace
     * @return HodnoceniSkoleni
     * @throws Exception
     */
    function nactiHodnoceni($id, $editace)
    {
        require_once('person/hodnoceniskoleni.php');

        $hodnoceni = new HodnoceniSkoleni($this ->database, $id);

        $pravo = self::vratPravoNaHodnoceni($this ->database, $id, $editace);
        $hodnoceni ->fPravo = $pravo;

        if (!$pravo  ||  ($editace  &&  $pravo < 2)) {
            throw new NeniPravoException();
        }

        return $hodnoceni;
    }

    /**
     * @param int  $id
     * @param bool $editace
     * @param bool $postValues
     * @return ViewObject
     * @throws Exception
     * @throws DBException
     */
    function pripravenyDotaznik($id, $editace, $postValues)
    {
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba! Není zadán Id dotazníku!'));

        $dotaznik = $this ->nactiHodnoceni($id, $editace);

        // kontrola, jestli se needituje uzavreny dotaznik
        if ($editace  &&  !$dotaznik ->jeEditovatelnyStav()) { // ukonceno, ukonceno-vraceno
            MessageManager::addErrorMessage( ronText('Nelze_Editovat_uzavrene_hodnoceni') );
            $editace = false;
        }

        // Nastavi hodnotitele dotazniku.
        if ($editace  &&  !$postValues) {
            if (!$dotaznik->nastavHodnotitele()) {
                return new MessageManagerViewObject(); // zobrazi chybove hlasky
            }
        }

        return new DotaznikViewObject($dotaznik, $editace, $postValues);
    }

    /** Provede ulozeni hodnoceni
     * @return Redirect|ViewObject
     * @throws Exception
     */
    function ulozHodnoceni()
    {
        $id = (int)$_GET['record'];
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba! Není zadán Id dotazníku!'));

        $dotaznik = $this ->nactiHodnoceni($id, true); // editace = true

        $dotaznik->fUlozenoVXML = null;
        $dotaznik->fPrepoctenoVXML = null;

        if ($dotaznik->fXMLSablona) {
            $dotaznik->vyplnDotaznikXML();
        }

        // pokud nebyl dotaznik ulozen uz v XML, nebo nema XML, tak jej ulozi nyni
        if (isset($dotaznik->fUlozenoVXML)) {
            MessageManager::addDebugMessage(StringUtils::fromUTF8('Uložení odpovědí proběhlo přímo v XML formuláři'));
            $ulozeno_result = $dotaznik->fUlozenoVXML;
        } else {
            if ($dotaznik->fXMLSablona) {
                MessageManager::addDebugMessage(StringUtils::fromUTF8('Uložení odpovědí neproběhlo v XML -> proběhne nyní'));
            }

            try {
                $ulozeno_result = $dotaznik->ulozVyplnenyDotaznik();
            } catch (RonException $e) {
                MessageManager::addErrorMessage($e ->getMessage(), $e ->fieldName, null, $e ->getErrorDescription());
                $ulozeno_result = false;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e ->getMessage());
                $ulozeno_result = false;
            }
        }

        if ($ulozeno_result) {
            MessageManager::addInfoMessage(ronText('Hodnoceni_ulozeno'));
        } else {
            return new DotaznikViewObject($dotaznik, true, true);
        }

        // test, jestli se ma zaroven s ulozenim vratit nebo uzavrit (stisknuto tlacitko "uzavrit")
        if (isset($_POST['uzavrit']) && $_POST['uzavrit']) {
            if ($dotaznik->fVyplnenoVse) {
                return $this ->uzavriHodnoceni($id);
            }

            if (!$dotaznik->fVyplnenoVse) {
                MessageManager::addErrorMessage( ronText('Nelze_uzavrit_nevyplnene_hodnoceni') );
            }

            // pouzit redirect ? ? ?
            return $this ->pripravenyDotaznik($id, true, false); // editace = true, postvalues = false (uz je ulozeno)

        } else {
            // jen ulozeni - zobrazi v readonly
            return new Redirect(null, '?menuid=skoleni&action=hodnoceni&record=' . $id);
        }
    }

    /**
     * Uzavreni dotazniku (kliknuti na tlacitko Odeslat nebo Vratit k doplneni)
     * @param int $id
     * @return Redirect
     * @throws DBException
     * @throws Exception
     */
    public function uzavriHodnoceni($id)
    {
        if (!$id) throw new Exception(StringUtils::fromUTF8('Chyba! Není zadán Id dotazníku!'));

        MessageManager::addDebugMessage("HodnoceniController::uzavriDotaznik id=$id ");

        $params = $this->database->createParameters();

        // zapise jmeno hodnoticiho a cas
        $sql =
            'UPDATE PripraveneTemaHodnoceni ' .
            "   SET HodnotilRC = " . $params ->addParameter(User::getRC()) . ", " .
            "       Hodnotici = " . $params ->addParameter(RonPortalUtils::getLogUserName()) . ", " .
            '       DatumHodnoceni = ' . $params->addParameter($this ->database ->timestampToParam(time())) . ', ' .
            '       Stav = 2 '  . //2 - hodnocení ukončeno
            ' WHERE Id = ' . $params->addParameter($id);

        $result = $this ->database ->sql($sql, $params);
        if (!$result) {
            throw new DBException($this ->database, $sql, StringUtils::fromUTF8('Chyba při nastavení hodnotitele!'));
        }

        return new Redirect(null, "?menuid=skoleni");
    }


    /**
     * @return UcastNaSkoleniBrowser
     * @throws Exception
     */
    private function zobrazUcastSkoleni()
    {
        HtmlHeaders::getInstance()->addScript('script/skoleni.js');

        $b = new UcastNaSkoleniBrowser($this->database);
        $b->load();
        return $b;
    }


    /**
     * @param $kodKurzu
     * @return HodnoceniUcastnikuBrowser
     * @throws Exception
     */
    private function zobrazHodnoceniUcastniku($kodKurzu)
    {
        if (!Prava::jsouNejakePrava('skoleni_absolvovani')) {
            throw new NeniPravoException();
        }

        $hodnoceniBrowser = new HodnoceniUcastnikuBrowser($this ->database, $kodKurzu);
        if (isset($_POST['HodnoceniUcastniku'])  &&  $_POST['HodnoceniUcastniku']) {
            try {
                $hodnoceniBrowser->ulozZmenyHodnoceni();
                return new Redirect(null, isset($_POST['BackButton'])
                    ?  '?menuid=skoleni&action=skolenikurzy'
                    : '?menuid=skoleni&action=skolenihodnoceni&record=' . $kodKurzu);
            } catch (DBException $e) {
                throw $e;
            } catch (Exception $e) {
                MessageManager::addErrorMessage($e);
            }
        } else {
            $hodnoceniBrowser->load();
        }
        return $hodnoceniBrowser;

//        // TODO: odstranit ....
//        require_once ('person/b_skolenihodnoceni.php');
//        $b = new HodnoceniSkoleniBrowser($this->database, $kodKurzu);
//        $b->load();
//        return $b;
    }


    /**
     * @return SkoleniAKurzyBrowser
     * @throws Exception
     */
    private function zobrazSkoleniKurzy()
    {
        $this->hlavickaVyberAkce();

        // prava "skoleni_lektor" kontroluje browser
        $b = new SkoleniAKurzyBrowser($this->database);
        $b->load();
        return $b;
    }

//    /**
//     * @param $timestampTerminuHodnoceni
//     * @return Redirect
//     * @throws Exception
//     */
//    private function zapisUcastiNaSkoleni($timestampTerminuHodnoceni)
//    {
//        $this->pravoNaZapisUcastAHodnoceni();
//
//        try {
//            $this->ulozZmenenyZaznam($this->database, $timestampTerminuHodnoceni);
//        } catch (Exception $e) {
//            MessageManager::addErrorMessage($e ->getMessage());
//        }
//
//        return new Redirect(null, '?menuid=skoleni&action=skoleniucast');
//    }

//    /**
//     * @return Redirect
//     * @throws Exception
//     */
//    private function zapisHodnoceniSkoleni()
//    {
//        $this->pravoNaZapisUcastAHodnoceni();
//
//        try {
//            $this->ulozZmenyHodnoceni($this->database);
//        } catch (Exception $e) {
//            MessageManager::addErrorMessage($e ->getMessage());
//        }
//
////        return new Redirect(null, '?menuid=skoleni&action=skolenihodnoceni&record=' . $cisloSkoleni);
//        return new Redirect(null, '?menuid=skoleni&action=skolenikurzy');
//    }

//    /**
//     * Ulozeni Ucasti na skoleni
//     * @param Database $database
//     * @param $timestampTerminuSkoleni
//     * @throws Exception
//     */
//    function ulozZmenenyZaznam($database, $timestampTerminuSkoleni)
//    {
//        $RcUcastnika = '';
//        $poleRCaOsC = $this->nactiRCaOsobniCislaUGenerovanehoSkoleni($database, $_SESSION['kodDlouhodobehoKurzu']);
//        $this->pravoNaZapisUcastAHodnoceni();
//
//        $druhSkoleni = $_SESSION['kodDlouhodobehoKurzu'];
//
//        for ($i = 1; $i <= $_POST['pocetRadku']; $i++) {
//
//            $idUcastnika_post =  $i . "_idUcastnika";
//            $idUcastnika = $_POST[$idUcastnika_post];
//
//            $poznamkaKUchazeci_post =  $i . "_poznamkaKUchazeci";
//            $poznamkaKUchazeci = $_POST[$poznamkaKUchazeci_post];
//
//            //Procházím pole účastníků, které se jednou načte z SQL
//            foreach ($poleRCaOsC as $osoba => $oc){
//                if($oc[0] == $idUcastnika) {
//                    $RcUcastnika = $oc[1];
//                    //Pokud je RC nalezeno ukončím foreach.
//                    break 1;
//                }
//            }
//
//            $pritomenNaTerminu_post =  $i . "_pritomenNaTerminu";
//            $pritomenNaTerminu = ($_POST[$pritomenNaTerminu_post] != "") ? $_POST[$pritomenNaTerminu_post] : "0";
//
//            $omluvenaUcast_post =  $i . "_omluvenaUcast";
//            $omluvenaUcast = ($_POST[$omluvenaUcast_post] != "") ? $_POST[$omluvenaUcast_post] : "0";
//
//
//            $puvodniHodnoty_post =  $i . "_idUcastnikaPuvodniHodnoty";
//            $puvodniHodnoty = explode(";;", $_POST[$puvodniHodnoty_post]);
//
//
//            $zmenaUcasti = false;
//            if (($puvodniHodnoty[0] != $pritomenNaTerminu) || ($puvodniHodnoty[1] != $omluvenaUcast) || ($puvodniHodnoty[2] != $poznamkaKUchazeci)){
//                $zmenaUcasti = true;
//            }
//
//            if ($zmenaUcasti)
//            {
//                $record = new RonPortalDBRecord($database, 'UcastNaTerminuSkoleni', $this);
//
//                $record->addField('CisloSkoleni', '', FALSE, 10, $druhSkoleni);
//                $record->addField('Datum', 'timestamp', FALSE, 0, $timestampTerminuSkoleni);
//                $record->addField('RC', '', FALSE, 15, $RcUcastnika);
//                $record->addField('Pritomen', 'int', FALSE, 0, $pritomenNaTerminu);
//                $record->addField('DuvodNepritomnosti', 'int', FALSE, 0, ($pritomenNaTerminu == 1) ? 0 : $omluvenaUcast);
//                $record->addField('Poznamka', 'memo', FALSE, 0, $poznamkaKUchazeci);
//
//                $params = $database->createParameters();
//                $record->setWhere( "RC = " . $params->addParameter($RcUcastnika) .
//                    " AND CisloSkoleni = " . $params->addParameter($druhSkoleni) .
//                    " AND Datum = " . $params->addParameter($this->database->dateToParam($timestampTerminuSkoleni)), $params);
//
//                if (!$record->updateRecord()) {
//                    throw new Exception(ronText('zmena_ucasti_na_terminu_neprobehla'));
//                }
//            }
//        }
//    }


    /**
     * @throws Exception
     */
    private function pravoNaZapisUcastAHodnoceni()
    {
        $prava = Prava::vratPrava('skoleni_absolvovani');

        if (!$prava->jsouNejakePrava()){
            throw new Exception(ronText('neni_pravo'));
        }
    }

    /**
     * @param Database $database
     * @param $kurz
     * @return array
     * @throws Exception
     */
    private function nactiRCaOsobniCislaUGenerovanehoSkoleni($database, $kurz)
    {
        $result = [];

        $params = $database->createParameters();

        $sql = "SELECT PracovniPomer.ID, PracovniPomer.RC, Osoba.Prijmeni " .
               " FROM GenerovaneSkoleni INNER JOIN Osoba ON (Osoba.RC = GenerovaneSkoleni.RC) INNER JOIN PracovniPomer ON (PracovniPomer.RC = GenerovaneSkoleni.RC) " .
               " WHERE GenerovaneSkoleni.CisloSkoleni = " . $params->addParameter($kurz) .
               " ORDER BY PracovniPomer.ID ";
        $cursor = $database->getSqlResult($sql, $params);

        while ($row = $cursor->fetch()){
            $result[] = $row;
        }
        $cursor->close();

        return $result;
    }

//    /**
//     * Ulozeni Hodnoceni
//     * @param Database $database
//     * @throws Exception
//     */
//    function ulozZmenyHodnoceni($database)
//    {
//        $RcUcastnika = '';
//        $druhSkoleni = $_POST['druhSkoleni'];
//        $cisloSkoleni = $_POST['cisloSkoleni'];
//
//        $poleRCaOsC = $this->nactiRCaOsobniCislaUGenerovanehoSkoleni($database, $cisloSkoleni);
//
//        for ($i = 1; $i <= $_POST['pocetRadku']; $i++)
//        {
//            //Datum vzniku povinnosti pro setWhere
//            $timestampDatumVznikuPovinnosti_post =  $i . "_datumVznikuPovinnosti";
//            $timestampDatumVznikuPovinnosti = (isset($_POST[$timestampDatumVznikuPovinnosti_post]) && $_POST[$timestampDatumVznikuPovinnosti_post] != "") ? $_POST[$timestampDatumVznikuPovinnosti_post] : "";
//
//            //ID pro vyhledani Rc
//            $idUcastnika_post =  $i . "_idUcastnika";
//            $idUcastnika = (isset($_POST[$idUcastnika_post]) && $_POST[$idUcastnika_post] != "") ? $_POST[$idUcastnika_post] : "";
//
//            //Hodnoceni daneho ucastnika
//            $hodnoceniUcastnika_post =  $i . "_hodnoceniCombo";
//            $hodnoceniUcastnika = $_POST[$hodnoceniUcastnika_post];
//
//            //Komentar hodnoceni ucastnika
//            $slovniHodnoceniUcastnika_post =  $i . "_slovniHodnoceni";
//            $slovniHodnoceniUcastnika = $_POST[$slovniHodnoceniUcastnika_post];
//
//            //Prochazim pole ucastniku, ktere se jednou nacte z SQL
//            foreach ($poleRCaOsC as $osoba => $oc){
//                if($oc[0] == $idUcastnika) {
//                    $RcUcastnika = $oc[1];
//                    //Pokud je RC nalezeno skoncit foreach.
//                    break 1;
//                }
//            }
//
//            $puvodniHodnoty_post =  $i . "_idUcastnikaPuvodniHodnotyHodnoceni";
//            $puvodniHodnoty = explode(";;", filter_input(INPUT_POST, $puvodniHodnoty_post, FILTER_SANITIZE_SPECIAL_CHARS));
//
//            $zmenaUcasti = false;
//            if (($puvodniHodnoty[0] != $hodnoceniUcastnika) || ($puvodniHodnoty[1] != $slovniHodnoceniUcastnika)){
//                $zmenaUcasti = true;
//            }
//
//            if ($zmenaUcasti) {
//                $record = new RonPortalDBRecord($database, 'GenerovaneSkoleni', $this);
//
//                $record->addField('RC', '', FALSE, 15, $RcUcastnika);
//                $record->addField('DruhSkoleni', '', FALSE, 10, $druhSkoleni);
//                $record->addField('DatumVznikuPovinnosti', 'timestamp', FALSE, 0, $timestampDatumVznikuPovinnosti);
//                $record->addField('Potvrzeni', 'int', FALSE, 0, $hodnoceniUcastnika);
//                $record->addField('Poznamka', 'memo', FALSE, 0, $slovniHodnoceniUcastnika);
//
//                $params = $this->database->createParameters();
//                $record->setWhere('RC = ' . $params->addParameter($RcUcastnika) .
//                    ' AND DruhSkoleni = ' . $params->addParameter($druhSkoleni) .
//                    ' AND DatumVznikuPovinnosti = ' . $params->addParameter($this->database->timestampToParam($timestampDatumVznikuPovinnosti)), $params);
//
//
//                if (!$record->updateRecord()) {
//                    throw new Exception(ronText('zmena_hodnoceni_neprobehla'));
//                }
//            }
//        }
//    }


//    /**
//     * @param $kodKurzu
//     * @param $datumKurzu
//     * @param $casKurzu
//     * @param $poznamkaKurzu
//     * @return Redirect
//     * @throws Exception
//     */
//    private function vytvoreniTerminuSkoleni($kodKurzu, $datumKurzu, $casKurzu, $poznamkaKurzu)
//    {
//        $prava = Prava::vratPrava('skoleni_absolvovani');
//
//        if (!$prava->jsouNejakePrava()){
//            throw new NeniPravoException();
//        }
//
//        //Převod z textu do timestamp
//        $datum = DateUtils::dateStringToTimestamp($datumKurzu);
//        $cas = DateUtils::timeStringToTimestamp($casKurzu);
//
//        $params = $this->database->createParameters();
//        //Test zdali je termin volny
//        $sql = "SELECT Datum FROM TerminSkoleni WHERE CisloSkoleni = " . $params->addParameter($kodKurzu) . " AND Datum = " . $params->addParameter($this->database->timestampToParam($datum));
//
//        $terminJeVolny = DBUtils::getSqlValue($this->database, $sql, $params);
//
//        if ($terminJeVolny == ""){
//            $_SESSION['terminJeObsazen'] = 0;
//            $_SESSION['poznamkaNovehoTerminu'] = "";
//
//            $record = new RonPortalDBRecord($this->database, 'TerminSkoleni', $this);
//
//            $record->addField('CisloSkoleni', '', FALSE, 10, $kodKurzu);
//            $record->addField('Datum', 'timestamp', FALSE, 0, $datum);
//            $record->addField('Cas', 'time', FALSE, 0, $cas);
//            $record->addField('Poznamka', 'memo', FALSE, 0, $poznamkaKurzu);
//
//            $record->insertRecord();
//                //MessageManager::addInfoMessage(ronText('vytvoreni_terminu_selhalo'));
//
//            //Pokud zapis uspesny - pripojit ucastniky zajezdu
//            $params = $this->database->createParameters();
//            $sql = "SELECT GS.RC " .           //0
//                "FROM GenerovaneSkoleni GS " .
//                "JOIN Osoba ON (Osoba.RC = GS.RC) " .
//                "LEFT JOIN TerminSkoleni ON (TerminSkoleni.CisloSkoleni=GS.CisloSkoleni) LEFT JOIN PracovniPomer On (Osoba.RC = PracovniPomer.RC)" .
//                "LEFT JOIN UcastNaTerminuSkoleni UTS ON (UTS.CisloSkoleni=TerminSkoleni.CisloSkoleni AND UTS.RC=GS.RC AND UTS.Datum=TerminSkoleni.Datum) " .
//                "WHERE (GS.CisloSkoleni = " . $params->addParameter($kodKurzu) . ") AND (GS.Potvrzeni<5)  AND TerminSkoleni.Datum = " . $params ->addParameter($this ->database ->timestampToParam($datum)) . " " .
//                " AND PracovniPomer.ID = (SELECT Max(PP.ID) FROM PracovniPomer PP Where PP.RC = GS.RC) " .
//                "ORDER BY RC";
//
//            $cursor = $this->database->getSqlResult($sql, $params);
//
//            while ($row = $cursor->fetch()) {
//
//                $record = new RonPortalDBRecord($this->database, 'UcastNaTerminuSkoleni', $this);
//
//                $record->addField('CisloSkoleni', '', FALSE, 10, $kodKurzu);
//                $record->addField('Datum', 'timestamp', FALSE, 0, $datum);
//                $record->addField('RC', '', FALSE, 15, $row[0]);
//                $record->addField('Pritomen', 'int', FALSE, 0, 0);
//                $record->addField('DuvodNepritomnosti', 'int', FALSE, 0, 0);
//                $record->addField('Poznamka', 'memo', FALSE, 0, "");
//
//                $record->insertRecord(); //  throw new Exception(ronText('pripojeni_na_termin_selhalo'));
//            }
//
//        } else {
//            $_SESSION['terminJeObsazen'] = 2;
//            $_SESSION['poznamkaNovehoTerminu'] = $poznamkaKurzu;
//        }
//        return new Redirect(null, '?menuid=skoleni&action=skoleniucast');
//    }

//    /**
//     * @param $kodKurzu     //Kod
//     * @param $datumKurzu   //datum
//     * @param $casKurzu        //Čas
//     * @param $poznamkaKurzu    //Poznamka
//     * @return Redirect
//     * @throws Exception
//     */
//    private function editaceTerminuSkoleni($kodKurzu, $datumKurzu, $casKurzu, $poznamkaKurzu)
//    {
//        $prava = Prava::vratPrava('skoleni_absolvovani');
//        Prava::nastavObdobiProPrava(time(), time());
//
//        if (!$prava->jsouNejakePrava()){
//            throw new Exception(ronText('Nemate_pravo'));
//        }
//
//        //Převod z textu do timestamp
////        $datum = DateUtils::dateStringToTimestamp($datumKurzu);
//        $cas = DateUtils::timeStringToTimestamp($casKurzu);
//
//        $_SESSION['terminJeObsazen'] = 0;
//        $_SESSION['poznamkaNovehoTerminu'] = "";
//
//        $record = new RonPortalDBRecord($this->database, 'TerminSkoleni', $this);
//
//        $record->addField('Cas', 'time', FALSE, 0, $cas);
//        $record->addField('Poznamka', 'memo', FALSE, 0, $poznamkaKurzu);
//
//        $params = $this->database->createParameters();
//
//        $record->setWhere(" CisloSkoleni = " . $params->addParameter($kodKurzu) .
//                         " AND Datum = " . $params->addParameter($this->database->timestampToParam($datumKurzu)), $params);
//
//        if (!$record->updateRecord()) {
//            MessageManager::addInfoMessage(ronText('editace_terminu_selhala'));
//        }
//
//        return new Redirect(null, '?menuid=skoleni&action=skoleniucast');
//    }


    /**
     * @return PlanovaneSkoleniBrowser
     * @throws Exception
     */
    private function zobrazPlanovaneSkoleni()
    {
        $prava = Prava::vratPrava('skoleni_plan');

        if (!$prava->jsouNejakePrava()){
            throw new NeniPravoException();
        }

        $this->hlavickaVyberAkce();

        if (isset($_GET['formaction'])) {
            return $this ->detailPlanovaneSkoleni();
        }

        $b = new PlanovaneSkoleniBrowser($this->database);
        $b->load();
        return $b;
    }


//    /**
//     * @param Database $database
//     * @param $selected
//     * @return mixed
//     * @throws DBException
//     */
//    private function comboTypuSkoleni($database, $selected = null)
//    {
//        $sql = "SELECT Kod, Popis FROM DruhSkoleni WHERE Aktivni = 1 ORDER BY Popis";
//
//        $combo = new ComboBox('druhSkoleni');
//        $combo->emptyRow = '';
//        $combo->alwaysShowSelected = isset($value) && $value != '';
//        $combo->htmlParams = "title='Typ skoleni'";
//        $combo->className .= 'adsfilterforminput_full nonempty';
//        $combo->emptyRow = ronText('nevybrano');
//        $combo->showCode = true;
//        $combo->loadData($database, $sql, $selected);
//
//        return $combo;
//    }


//    /**
//     * @param Database $database
//     * @param $selected
//     * @param PlanovaneSkoleniObjekt |null $hodnotyPlanu
//     * @return mixed
//     * @throws DBException
//     * @throws Exception
//     */
//    private function comboStavuSkoleni($database, $selected = null, $hodnotyPlanu = null)
//    {
//        $sqlWhere = '';
//        $prava = Prava::vratPrava('skoleni_schvaleni_planu');
//        if (Prava::vratHodnotuPrava('skoleni_schvaleni_planu') != 3)
//        {
//            //Pokud nemá právo na vše, pak testovat zda může upravovat zvolené
//            if ($selected === null)
//            {
//                //Při Insert testovat zda mám právo na přihlášenou osobu na portále
//                if ($prava->lzeZobrazitRC(User::getRC(), time(), time()))
//                {
//                    //Má právo na Schválení k přihlašování pro sebe (nemůže schvalovat jiné - ale sebe ano) - přístupny všechny možnosti
//                    $selected = 0;
//                } else {
//                    //Nemá právo na Schválení k přihlašování (combo má tedy jedinou možnost - Požadavek)
//                    $selected = -1;
//                    $sqlWhere = "Where Kod IN (-1)";
//                }
//
//            } else {
//                //Edit - existuje alespoň jedna přihlášená osoba - autor, testovat zda mám právo na autora
//                //Autor 1. osoba na kurz přihlášena - mela by být pouze 1, tedy MIN(DatumVlozeni)
//                $params = $database->createParameters();
//
//                $sql = "SELECT RC FROM PlanovaneSkoleniOsob WHERE (CisloSkoleni = " . $params->addParameter($hodnotyPlanu->cisloSkoleni) . ")
//                     AND DatumVlozeni = (Select MIN(DatumVlozeni) FROM PlanovaneSkoleniOsob WHERE (CisloSkoleni = " . $params->addParameter($hodnotyPlanu->cisloSkoleni) . ") )";
//
//                $autorPlanu = $database->getSqlResult($sql, $params);
//                $row = $autorPlanu->fetch();
//                if ($prava->lzeZobrazitRC($row[0], time(), time()))
//                {
//                    //Má právo na Schválení k přihlašování - a může tedy zmenit stav
//                    $sqlWhere = '';
//
//                } else {
//
//                    if ($selected > -1) {
//                        //Nemá právo na Schválení k přihlašování ale Schválení k přihlašování je nastaveno - nemenit
//                        $sqlWhere = "Where Kod IN ($selected)";
//                    } else {
//                        //Nemá právo na Schválení k přihlašování a proto zůstane Požadavek
//                        $sqlWhere = "Where Kod IN (-1)";
//                    }
//                }
//            }
//        }
//
//        $sql = "SELECT Kod, Popis FROM S_StavPlanovanehoSkoleni {$sqlWhere}";
//
//        $combo = new ComboBox('stav');
//        $combo->alwaysShowSelected = isset($value) && $value != '';
//        $combo->htmlParams = "title='Stav skoleni'";
//        $combo->className .= 'adsfilterforminput_full';
//        $combo->loadData($database, $sql, $selected);
//
//        return $combo;
//    }

//    /**
//     * @param Database $database
//     * @param $firma
//     * @return ComboBox
//     * @throws DBException
//     */
//    public function comboFirmaSkoleni($database, $firma=null)
//    {
//        $params = $this->database->createParameters();
//        $sqlWhere = $firma != "" ? ("Where kodFirmy = " . $params->addParameter($firma)) : "";
//
//        $sql = "SELECT kodFirmy AS Kod, COALESCE(nazev1  {fn_concat()} ' (' {fn_concat()} nazev2 {fn_concat()} ')', nazev1) AS Popis FROM Firma {$sqlWhere} ORDER BY Popis";
//
//        $combo = new ComboBox('skoliciFirma');
//        $combo->emptyRow = ronText('nevybrano');
//        $combo->alwaysShowSelected = isset($value) && $value != '';
//        $combo->htmlParams = "title='Skolici firma'";
//        $combo->className .= 'adsfilterforminput_full';
//        $combo->loadData($database, $sql, $firma, $params);
//
//        return $combo;
//    }


//    /**
//     * @param Database $database
//     * @param $cisloSkoleni
//     * @return array
//     * @throws Exception
//     */
//    public function comboNakladuSkoleni($database, $cisloSkoleni)
//    {
//        $comboNakladovePolozky = [];
//        $params = $database->createParameters();
//
//        $sql = "SELECT TP.Kod, TP.Popis, TP.SazbaDPH
//                FROM TypPolozky TP
//
//                WHERE (NakladNaTermin = 0)
//                AND NOT (EXISTS (SELECT TypPolozky FROM PlanovanaCenaSkolCelkova WHERE CisloSkoleni = " . $params->addParameter($cisloSkoleni) . "
//                AND TP.kod = PlanovanaCenaSkolCelkova.TypPolozky))
//                ORDER BY Popis";
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        while ($row = $cursor->fetch()){
//            $nakladovaPolozka = [];
//            $nakladovaPolozka['kod'] = $row[0];
//            $nakladovaPolozka['popis'] = $row[1];
//            $nakladovaPolozka['sazbaDPH'] = $row[2];
//
//            $comboNakladovePolozky[] = $nakladovaPolozka;
//        }
//
//        return $comboNakladovePolozky;
//    }

//    /**
//     * @param $cisloSkoleni
//     * @return array
//     * @throws Exception
//     */
//    public function nakladovePolozkyPlanovanehoSkoleni($cisloSkoleni)
//    {
//        $params = $this->database->createParameters();
//
//        $nakladovePolozky = [];
//        $sql =
//            "SELECT PC.CisloSkoleni, PC.TypPolozky, PC.SazbaDPH, PC.DPH, PC.CenaSDPH, TypPolozky.Popis
//             FROM PlanovanaCenaSkolCelkova PC LEFT JOIN TypPolozky ON (TypPolozky.Kod = PC.TypPolozky)
//             WHERE PC.CisloSkoleni = " . $params->addParameter($cisloSkoleni) .
//            " ORDER BY Popis"    ;
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        while ($row = $cursor->fetch()){
//            $nakladovaPolozka = [];
//            $nakladovaPolozka['cisloSkoleni'] = $row[0];
//            $nakladovaPolozka['typPolozky'] = $row[1];
//            $nakladovaPolozka['sazbaDPH'] = $row[2];
//            $nakladovaPolozka['DPH'] = $row[3];
//            $nakladovaPolozka['cenaSDPH'] = $row[4];
//            $nakladovaPolozka['popis'] = $row[5];
//
//            $cenaBezDPH = 0;
//            if($row[2] == 0){
//                //Pokud neni DPH pak je cena bez DPH jako s DPH
//                $cenaBezDPH = $row[4];
//            }
//
//            if ($row[2] > 0  && $row[4] > 0){
//                $koeficient = $row[2] / ($row[2]+100);
//                $cenaBezDPH = $row[4]  - ($koeficient * $row[4]);
//            }
//
//            $nakladovaPolozka['cenaBezDPH'] = $cenaBezDPH;
//
//            $nakladovePolozky[] = $nakladovaPolozka;
//        }
//
//        return $nakladovePolozky;
//    }

    /**
     * @return ViewObject
     * @throws Exception
     */
    public function detailPlanovaneSkoleni() // $cisloSkoleni, $nactenyPost = null)
    {
        $prava = Prava::vratPrava('skoleni_plan');
        if (!$prava ->jsouNejakePrava()) {
            throw new NeniPravoException();
        }

//        $pp = PracovniPomer::nactiPPOsoby($this ->database, User::getRC());
//
//        $params = $this->database->createParameters();
//        $sql = "Select RC From PlanovaneSkoleniOsob
//                            Where DatumVlozeni = (Select MIN(DatumVlozeni) From PlanovaneSkoleniOsob Where PlanovaneSkoleniOsob.CisloSkoleni = " . $params->addParameter($cisloSkoleni) .")";
//        $autorPlanu = DBUtils::getSqlValue($this->database, $sql, $params);
//
//        if (!$prava->lzeZobrazitRC($autorPlanu)){
//            throw new NeniPravoException();
//        }

        //HtmlHeaders::getInstance()->addScript('script/skoleniucast.js');

        $form = new PlanovaneSkoleniForm($this ->database);
        if ($form ->handleAction()) {
            return new Redirect(null, '?menuid=skoleni&action=planovaneskolenikurzy');
        }

        return $form;

//        $nakladovePolozky = $this->nakladovePolozkyPlanovanehoSkoleni($cisloSkoleni);
//
//        require_once ('person/planovaneskoleni.php');
//        $detailPlanovanehoSkoleni = PlanovaneSkoleniObjekt::nactiPlanovaneSkoleni($this->database, $cisloSkoleni);
//
//        Model::addData("pp", $pp);
//        Model::addData("nactenyPost", $nactenyPost);
//        Model::addData("nakladovePolozky", $nakladovePolozky);
//
//        Model::addData('comboNakladuSkoleni', $this->comboNakladuSkoleni($this->database, $cisloSkoleni));
//        Model::addData('comboFirmaSkoleni', $this->comboFirmaSkoleni($this->database, $detailPlanovanehoSkoleni->firma));
//        Model::addData("detailPlanovanehoSkoleni", $detailPlanovanehoSkoleni);
//
//        Model::addData('comboTypuSkoleni', $this->comboTypuSkoleni($this->database, $nactenyPost != "" ? $nactenyPost->druhSkoleni : $detailPlanovanehoSkoleni->druhSkoleni));
//        Model::addData('comboStavuSkoleni', $this->comboStavuSkoleni($this->database, $detailPlanovanehoSkoleni->stav, $detailPlanovanehoSkoleni));
//        return new  BaseViewObject("person/view_vytvoreniskoleni.php");
    }

//    /**
//     * @param PlanovaneSkoleniObjekt|null $nactenyPost
//     * @return BaseViewObject
//     * @throws Exception
//     */
//    public function vytvoreniPlanovanehoSkoleni($nactenyPost = null)
//    {
//        $prava = Prava::vratPrava('skoleni_plan');
//
//        if (!$prava->jsouNejakePrava()){
//            throw new NeniPravoException();
//        }
//        $this->hlavickaVyberAkce();
//
//        //HtmlHeaders::getInstance()->addScript('script/skoleniucast.js');
//
//        $params = $this->database->createParameters();
//        $id = DBUtils::getSqlValue($this->database, "SELECT MAX(ID) FROM PracovniPomer WHERE RC = " . $params->addParameter(User::getRC()), $params);
//
//        $pp = new PracovniPomer();
//        $pp->nactiPPPodleId($this->database, $id);
//
//        Model::addData("pp", $pp);
//        Model::addData("nactenyPost", $nactenyPost);
//
//        Model::addData('comboFirmaSkoleni', $this->comboFirmaSkoleni($this->database ));
//        Model::addData('comboTypuSkoleni', $this->comboTypuSkoleni($this->database, $nactenyPost ? $nactenyPost->druhSkoleni : null));
//        Model::addData('comboStavuSkoleni', $this->comboStavuSkoleni($this->database));
//        return new  BaseViewObject("person/view_vytvoreniskoleni.php");
//    }

//    /** Planovene skoleni ...
//     * @return stdClass
//     */
//    public function nactiPOSTProDialog()
//    {
//        $hodnoty = new stdClass();
//        foreach ($_POST as $key => $value) {
//
//            switch ($key){
//                case "datumPocatkuSkoleni":
//                case "datumUkonceniSkoleni":
//                    try{
//                        if ($value != "") {
//                            $hodnoty->$key = DateUtils::dateStringToTimestamp($value);
//                        }
//                    } catch (Exception $e) {
//                        MessageManager::addDebugLog($e, $key);
//                        $hodnoty->$key = null;
//                    }
//                    break;
//
//                case "casPocatkuSkoleni":
//                case "casUkonceniSkoleni":
//                    try{
//                        if ($value != "") {
//                            $hodnoty->$key = DateUtils::timeStringToTimestamp($value, $hodnoty->datumPocatkuSkoleni);
//                        }
//                    } catch (Exception $e) {
//                        MessageManager::addDebugLog($e, $key);
//                        $hodnoty->$key = null;
//                    }
//                    break;
//
//
//                default:
//                    $hodnoty->$key = $value;
//            }
//        }
//
//        return $hodnoty;
//    }


//    /**
//     * @param $cisloSkoleni
//     * @return bool
//     * @throws Exception
//     */
//    private function shodaCislaSkoleni($cisloSkoleni)
//    {
//        $params = $this->database->createParameters();
//        $sql = "SELECT CisloSkoleni FROM PlanovaneSkoleni WHERE CisloSkoleni = " . $params->addParameter($cisloSkoleni);
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        $row = $cursor->fetch();
//
//        if ($row[0])
//            return true;
//
//        return false;
//    }

//    /**
//     * @throws Exception
//     */
//    public function upravaNakladuPlanovanehoSkoleni()
//    {
//        $cnt = 0;
//        $polePolozek = [];
//        $pocet_polozek = $_POST['pocet_polozek'];
//        for ($i = 0; $i <= $pocet_polozek; $i++)
//        {
//            $polozka = "polozka". $cnt;
//
//            $polePolozek[$cnt]['nakladovaPolozka'] = $_POST[$polozka . '_nakladovaPolozka'];
//            $polePolozek[$cnt]['cenaBezDPH'] = $_POST[$polozka . '_cenaBezDPH'];
//            $polePolozek[$cnt]['sazbaDPH'] = $_POST[$polozka . '_sazbaDPH'];
//            $polePolozek[$cnt]['cenaSDPH'] = $_POST[$polozka . '_cenaSDPH'];
//            ++$cnt;
//        }
//
//        foreach ($polePolozek as $polozka=>$hodnota)
//        {
//            $cisloSkoleni = StringUtils::htmlTextConvert($_POST['cisloSkoleni']);
//            $sazbaDPH = $hodnota['sazbaDPH'];
//            $vypocteneDPH = ($hodnota['cenaBezDPH'] / 100) * $sazbaDPH;
//            $cenaSDPH = $hodnota['cenaBezDPH'] + $vypocteneDPH;
//
//            $rec = new RonPortalDBRecord($this->database, 'PlanovanaCenaSkolCelkova', $this);
//            $rec->addField('CisloSkoleni', '', false, 10, $cisloSkoleni);
//            $rec->addField('TypPolozky', '', false, 10, $hodnota['nakladovaPolozka']);
//            $rec->addField('SazbaDPH', 'float', false, 0, $sazbaDPH);
//            $rec->addField('DPH', 'float', false, 0, $vypocteneDPH);
//            $rec->addField('CenaSDPH', 'float', false, 0, $cenaSDPH);
//
//            $params = $this->database->createParameters();
//            $sqlWhere =
//                ' CisloSkoleni = ' . $params->addParameter($cisloSkoleni) .
//                ' AND TypPolozky = ' . $params->addParameter($hodnota['nakladovaPolozka']);
//            $rec->setWhere($sqlWhere, $params);
//            $rec ->updateOrInsert();
//        }
//    }


//    /**
//     * @throws Exception
//     */
//    public function noveNakladyPlanovanehoSkoleni()
//    {
//        $hodnotyPolozky = explode(";;", $_POST['pridatTypPolozky']);
//
//        $rec = new RonPortalDBRecord($this ->database, 'PlanovanaCenaSkolCelkova', $this);
//        $rec ->addField('CisloSkoleni', '', false, 10, $_POST['cisloSkoleni']);
//        $rec ->addField('TypPolozky', '', false, 10, $hodnotyPolozky[0]);
//        $rec ->addField('SazbaDPH', 'float', false, 0, $hodnotyPolozky[1]);
//        $rec ->addField('DPH', 'float', false, 0, 0);
//        $rec ->addField('CenaSDPH', 'float', false, 0, 0);
//
//        $params = $this->database->createParameters();
//        $sqlWhere =
//            ' CisloSkoleni = ' . $params->addParameter($_POST['cisloSkoleni']) .
//            ' AND TypPolozky = ' . $params->addParameter($hodnotyPolozky[0]);
//        $rec->setWhere($sqlWhere, $params);
//        $rec ->updateOrInsert();
//    }


//    /** Planovane skoleni ...
//     * @param $cisloSkoleni
//     * @return mixed
//     * @throws DBException
//     */
//    private function pocetOsobNaKurzu($cisloSkoleni)
//    {
//        $params = $this->database->createParameters();
//        $sql = "SELECT COUNT(RC) FROM PlanovaneSkoleniOsob PSO Where PSO.Potvrzeni = 1 AND PSO.CisloSkoleni = " . $params->addParameter($cisloSkoleni);
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        $row = $cursor->fetch();
//
//        return $row[0];
//    }


//    /**
//     * @param $cislo
//     * @return BaseViewObject|Redirect
//     * @throws Exception
//     */
//    public function zapisPlanovanehoSkoleni($cislo)
//    {
//        /** @var PlanovaneSkoleniObjekt $hodnoty */
//        $hodnoty = $this->nactiPOSTProDialog();
//        if ($_POST['cisloSkoleni'] == "") {
//            if ($this->shodaCislaSkoleni($hodnoty->cisloSkoleni)) {
//                MessageManager::addInfoMessage(ronText('cislo_skoleni_existuje'));
//                return $this->vytvoreniPlanovanehoSkoleni($hodnoty);
//            }
//        }
//
////        if (isset($_POST['nakladyplanovanehoskoleni']) && isset($_POST['nakladyplanovanehoskoleni']) != "" && $_POST['cisloSkoleni'] != "") {
////            $this->upravaNakladuPlanovanehoSkoleni();
////            return $this->detailPlanovaneSkoleni($_POST['cisloSkoleni']);
////        }
////
////
////        if (isset($_POST['novenakladyplanovanehoskoleni']) && isset($_POST['novenakladyplanovanehoskoleni']) != "" && $_POST['cisloSkoleni'] != "") {
////            $this->noveNakladyPlanovanehoSkoleni();
////            return $this->detailPlanovaneSkoleni($_POST['cisloSkoleni']);
////        }
//
//
//        $pocetPrihlasenychNaKurz = $this->pocetOsobNaKurzu($cislo);
//        if ($hodnoty->pocetOsob >= $pocetPrihlasenychNaKurz) {
//            //Nelze snizit pocet osob, pod prihlaseny pocet
//
//
//            $rec = new RonPortalDBRecord($this->database, 'PlanovaneSkoleni', $this);
//            $rec->addField('CisloSkoleni', '', false, 10, $hodnoty->cisloSkoleni);
//            $rec->addField('Popis', '', false, 160, $hodnoty->popis);
//            $rec->addField('DruhSkoleni', '', false, 10, $hodnoty->druhSkoleni);
//            $rec->addField('Stav', 'int', false, 0, $hodnoty->stav != "" ? $hodnoty->stav : -1);
//
//
//            $rec->addField('PocetOsob', 'int', false, 0, $hodnoty->pocetOsob);
//            $rec->addField('KonkretniUcastnici', 'int', false, 0, $hodnoty->konkretniUcastnici != 1 ? 0 : 1);
//            $rec->addField('DelkaSkoleniDnu', 'float', false, 0, $hodnoty->delkaSkoleniDnu);
//            $rec->addField('DelkaSkoleniHodin', 'float', false, 0, $hodnoty->delkaSkoleniHodin);
//            $rec->addField('DatumPocatkuSkoleni', 'timestamp', false, 0, $hodnoty->datumPocatkuSkoleni != "" ? $hodnoty->datumPocatkuSkoleni : null);
//            $rec->addField('CasPocatkuSkoleni', 'timestamp', false, 0, $hodnoty->casPocatkuSkoleni != "" ? $hodnoty->casPocatkuSkoleni : null);
//            $rec->addField('DatumUkonceniSkoleni', 'timestamp', false, 0, $hodnoty->datumUkonceniSkoleni != "" ? $hodnoty->datumUkonceniSkoleni : null);
//            $rec->addField('CasUkonceniSkoleni', 'timestamp', false, 0, $hodnoty->casUkonceniSkoleni != "" ? $hodnoty->casUkonceniSkoleni : null);
//            $rec->addField('Poznamka', 'memo', false, 0, $hodnoty->poznamka);
//            $rec->addField('SkoliciFirma', '', false, 10, $hodnoty->firma);
//
//            $rec->addField('Insert_UserName', '', true, 25, RonPortalUtils::getLogUserName());
//            $rec->addField('Insert_Timestamp', 'timestamp', true, 0, time());
//            $rec->addField('Update_UserName', '', false, 25, RonPortalUtils::getLogUserName())
//                ->updateLog = true;
//            $rec->addField('Update_Timestamp', 'timestamp', false, 0, time())
//                ->updateLog = true;
//
//            $params = $this->database->createParameters();
//            $sqlWhere =
//                ' CisloSkoleni = ' . $params->addParameter($hodnoty->cisloSkoleni);
//            $rec->setWhere($sqlWhere, $params);
//
//            if (!$rec->loadRecord()) {
//                $rec->insertRecord();
//                $this->pripojeniAutoraPlanovanehoKurzu($hodnoty);
//            } else {
//                $rec ->updateRecord();
//            }
//
//        } else {
//            MessageManager::addErrorMessage(sprintf(ronText('nelze_snizit_pocet_osob_pod_prihlasene_mnozstvi'), $pocetPrihlasenychNaKurz));
//        }
//
//
//
//        //Pokud ok redirect na detail
//        return new Redirect(null, '?menuid=skoleni&action=detailplanovanehoskolenikurzu&record=' . StringUtils::htmlTextConvert($cislo) );
//    }


    /**
     * @param $hodnoty
     * @throws DBException
     * @throws Exception
     */
    public function pripojeniAutoraPlanovanehoKurzu($hodnoty)
    {
        //připojit User na školení - jako autora a pokud má právo na přihlášení dané osoby tak nastavit potvrzení na 1
        $pravoPrihlasit = Prava::vratPrava('skoleni_prihlasit');

        $potvrzeni = 0;
        if ($pravoPrihlasit->lzeZobrazitRC(User::getRC())){
            $potvrzeni = 1;
        }

        $recZapis = new RonPortalDBRecord($this->database, 'PlanovaneSkoleniOsob', $this);

        $recZapis->addField('CisloSkoleni', '', false, 10, $hodnoty->cisloSkoleni);
        $recZapis->addField('RC', '', false, 15, User::getRC());
        $recZapis->addField('Potvrzeni', 'int', false, 0, $potvrzeni);
        $recZapis->addField('Vlozil', '', false, 25, User::getRC());
        $recZapis->addField('DatumVlozeni', 'timestamp', false, 0, time());
        $recZapis->addField('Vyhodnotil', '', false, 25, '');
        $recZapis->addField('DatumVyhodnoceni', 'timestamp', false, 0, '');
        $recZapis->addField('PrihlasenoZAplikace', 'int', false, 0, 1);

        $params = $this->database->createParameters();
        $recZapis->setWhere(
            'RC  = ' . $params->addParameter(User::getRC()) .
            ' AND CisloSkoleni = ' . $params->addParameter($hodnoty->cisloSkoleni), $params);

        if (!$recZapis->loadRecord()) {
            $recZapis->insertRecord();
            MessageManager::addInfoMessage(StringUtils::fromUTF8('Byl/a jste na plánované školení přihlášen/a.'));
        } else {
            MessageManager::addInfoMessage(StringUtils::fromUTF8('Již jste na plánované školení přihlášen/a.'));
        }
    }

    /** Prehled skoleni pro dashboard
     * @param JsonOutputObject $output
     * @throws DBException
     */
    private function mojeSkoleni($output)
    {
        $params = $this ->database ->createParameters();
        $sql =
            "SELECT DruhSkoleni.Kod, COALESCE(Skoleni.DatumPocatkuSkoleni, GS.PozadovaneDatumSkoleni), 
                    DruhSkoleni.Popis, Skoleni.DatumPocatkuSkoleni, GS.PozadovaneDatumSkoleni
			, CASE WHEN Skoleni.DatumPocatkuSkoleni IS NOT NULL THEN 1 ELSE 0 END
			, Skoleni.Popis
                FROM GenerovaneSkoleni GS 
                    LEFT JOIN DruhSkoleni ON (DruhSkoleni.Kod = GS.DruhSkoleni) 
                    LEFT JOIN Skoleni ON  (Skoleni.CisloSkoleni = GS.CisloSkoleni) 
                WHERE  (GS.RC= " . $params->addParameter(User::getRC()) .
            "  AND (GS.Potvrzeni=0 OR  (GS.Potvrzeni = 1  AND (Skoleni.StavSkoleni IS NULL  OR  Skoleni.StavSkoleni <> 2)) )
                OR GS.Potvrzeni=5 OR GS.Potvrzeni=6) " .
            ' ORDER BY COALESCE(Skoleni.DatumPocatkuSkoleni, GS.PozadovaneDatumSkoleni)';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=skoleni">';

        $html .=
            '<tr><th>' . ronText('Skoleni_termin_platnost') . '&nbsp;&nbsp;&nbsp;' .
            '</th><th>' . ronText('Skoleni_nazev_skoleni') .
            '</th><th>' . ronText('Skoleni_termin_povinnosti') . 
            '</th><th>' . ronText('Skoleni_prihlasen_na_kurz') . 
            '</th><th>' . ronText('Skoleni_popis_kurzu') . 
	    '</th><th></th></tr>';


        while ($row = $cursor ->fetch()) {
            MessageManager::addDebugMessage($row[3]);
            $html .=
                '<tr>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[1])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[2]) . '</td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[4])) . '</td>' .
                '<td>' . StringUtils::format('anone', $row[5]) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[6]) . '</td>' .
                '</tr>';
        }

        $html .= '</table>';

        $cursor ->close();

        $output ->addText('html', $html);
        $output->addDebugMessage($sql);
    }


    /** Prehled skoleni pro dashboard
     * @param JsonOutputObject $output
     * @throws Exception
     */
    private function skoleniPodrizenych($output)
    {
        $pravo = Prava::nactiPravaUzivatele($this->database,null,'skoleni_prohlizeni');

        $params = $this->database->createParameters();
        $html =
            '<table class="dashboardTable" data-ron-href="?menuid=skoleni">' .
            '<tr><th>' . ronText('Zamestnanec') .
            '</th><th>' . ronText('Skoleni_termin_platnost') . '&nbsp;&nbsp;' .
            '</th><th>' . ronText('Skoleni_nazev_kurzu_skoleni') .
            '</th><th>' . ronText('Skoleni_termin_povinnosti') .
            '</th><th></th></tr>';

        $sql =
            "SELECT DruhSkoleni.Kod, COALESCE(Skoleni.DatumPocatkuSkoleni, GenerovaneSkoleni.PozadovaneDatumSkoleni), 
                    DruhSkoleni.Popis, Osoba.Prijmeni, Osoba.Jmeno, Skoleni.DatumPocatkuSkoleni, GenerovaneSkoleni.PozadovaneDatumSkoleni 
                FROM (  (  ( GenerovaneSkoleni INNER JOIN Osoba ON (Osoba.RC = GenerovaneSkoleni.RC) INNER JOIN DruhSkoleni ON GenerovaneSkoleni.DruhSkoleni=DruhSkoleni.Kod) 
                INNER JOIN S_DuvodSkoleni ON GenerovaneSkoleni.DuvodSkoleni=S_DuvodSkoleni.Kod) 
                INNER JOIN S_StavPotvrzeni ON GenerovaneSkoleni.Potvrzeni=S_StavPotvrzeni.Kod) 
                LEFT OUTER JOIN Skoleni ON Skoleni.CisloSkoleni = GenerovaneSkoleni.CisloSkoleni 
                LEFT OUTER JOIN KategorieSkoleni ON DruhSkoleni.Kategorie=KategorieSkoleni.Kod 
                WHERE  (GenerovaneSkoleni.RC = Osoba.RC  AND (GenerovaneSkoleni.Potvrzeni=0 OR  (GenerovaneSkoleni.Potvrzeni = 1  AND (Skoleni.StavSkoleni IS NULL  OR  Skoleni.StavSkoleni <> 2)) )
                OR GenerovaneSkoleni.Potvrzeni=5 OR GenerovaneSkoleni.Potvrzeni=6) " .


        '  AND EXISTS(SELECT PracovniPomer.RC FROM PracovniPomer ' .
            '             WHERE PracovniPomer.RC = GenerovaneSkoleni.RC ' .
            '               AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) .
            '               AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->timestampToParam(DateUtils::today())) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
            '               AND ' .  $pravo->sqlFiltr('PracovniPomer', DateUtils::today(), DateUtils::today()) .
            '            )' .

            '   AND GenerovaneSkoleni.RC <> ' . $params->addParameter(User::getRC()) .

            ' ORDER BY COALESCE(Skoleni.DatumPocatkuSkoleni, GenerovaneSkoleni.PozadovaneDatumSkoleni)';

        $cursor = $this ->database ->getSqlResult($sql, $params);
        while ($row = $cursor ->fetch()) {
            $html .=
                '<tr>' .
                '<td>' . StringUtils::htmlTextConvert($row[3]) . ' ' . StringUtils::htmlTextConvert($row[4]) . '</td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[1])) . '</td>' .
                '<td>' . StringUtils::htmlTextConvert($row[2]) . '</td>' .
                '<td>' . StringUtils::format('finedate', $cursor ->stringToTimestamp($row[6])) . '</td>' .
                '</tr>';
        }
        $cursor ->close();

        $html .= '</table>';

        $output ->addText('html', $html);
    }


//    /**
//     * @return ComboBoxZamestnanec
//     */
//    public function osobaVyslanaNaSkoleni()
//    {
//        $url = "?menuid=skoleni&action=vyslatnaskoleni";
//
//        $prava = Prava::vratPrava('skoleni_vyslat');
//        Prava::nastavObdobiProPrava(DateUtils::today(), DateUtils::today());
//
//        $comboOsoby = new ComboBoxZamestnanec('osoba');
//        $comboOsoby->className .= 'form-input nonempty';
//        $comboOsoby->alwaysShowSelected = false;
//        $comboOsoby->htmlParams = "title='" . ronText('skoleni_osoba_na_skoleni') . "'";
//        $comboOsoby->htmlParams .= 'onChange="window.location.href = &quot;' . $url . '&amp;osoba=&quot; + encodeURIComponent(this.value)"';
//        $comboOsoby->showCode = 1;
//        $comboOsoby->loadDataPP($this->database, time(), time(), $prava, ronText('nevybrano'), '');
//
//        return $comboOsoby;
//    }


//    /**
//     * @param Database $database
//     * @param int $osoba
//     * @param $cisloSkoleni
//     * @return ComboBox
//     * @throws DBException
//     */
//    private function comboTypSkoleni($database, $osoba, $cisloSkoleni)
//    {
//        $sql = "SELECT Kod, Popis FROM DruhSkoleni WHERE Aktivni = 1 ORDER BY Popis";
//
//        $url = "?menuid=skoleni&action=vyslatnaskoleni&osoba=" . $osoba;
//
//        $combo = new ComboBox('druhSkoleni');
//        $combo->alwaysShowSelected = isset($value) && $value != '';
//        $combo->className .= 'form-input nonempty';
//        $combo->htmlParams = "title='" . ronText('Skoleni_typ_skoleni') . "'";
//        $combo->htmlParams .= 'onChange="window.location.href=&quot;' . $url . '&amp;druhskoleni=&quot; + encodeURIComponent(this.value)"';
//        $combo->emptyRow = ronText('nevybrano');
//        $combo->showCode = true;
//        $combo->loadData($database, $sql, $cisloSkoleni);
//
//        return $combo;
//    }

//    /**
//     * @param Database $database
//     * @param int $osoba
//     * @param string $druhSkoleni
//     * @param string $selected
//     * @return ComboBox
//     * @throws DBException
//     */
//    private function comboKurz($database, $osoba, $druhSkoleni, $selected = null)
//    {
//        $params = $database->createParameters();
//        $sql =
//            'SELECT CisloSkoleni, Popis FROM Skoleni' .
//            ' WHERE (StavSkoleni IN (0,1))' .
//            (isset($druhSkoleni)
//                ? ' AND DruhSkoleni = ' . $params->addParameter($druhSkoleni)
//                : ''
//            ) .
//            '  ORDER BY Popis';
//
//        $url = "?menuid=skoleni&action=vyslatnaskoleni&osoba=" . $osoba . "&druhskoleni=" . $druhSkoleni;
//
//        $combo = new ComboBox('kurz');
//        $combo->readOnly = (bool)!$druhSkoleni;
//        $combo->alwaysShowSelected = isset($value) && $value != '';
//        $combo->className .= 'form-input';
//        $combo->htmlParams = 'onChange="window.location.href=&quot;' . $url . '&amp;kurz=&quot; + encodeURIComponent(this.value)"';
//        $combo->emptyRow = ronText('nevybrano');
//        $combo->showCode = true;
//        $combo->loadData($database, $sql, $selected, $params);
//
//        return $combo;
//    }


//    /**
//     * @param $kurz
//     * @return array|bool
//     * @throws DBException
//     */
//    public function naplnenostKurzu($kurz)
//    {
//        if (!$kurz) {
//            return [null, null, -1];
//        }
//
//        $params = $this->database->createParameters();
//
//        $sql =
//            'SELECT Skoleni.Kapacita,  Count(*)' .
//            ' FROM Skoleni LEFT JOIN GenerovaneSkoleni GS' .
//            '  ON (Skoleni.CisloSkoleni = GS.CisloSkoleni) AND (GS.Potvrzeni < 5)' .
//            ' WHERE Skoleni.CisloSkoleni = ' . $params->addParameter($kurz) .
//            ' GROUP BY GS.DruhSkoleni, GS.CisloSkoleni, Skoleni.Kapacita';
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//        $row = $cursor->fetch();
//
//        if ($row){
//            return [
//                (int)$row[0],   // kapacita
//                (int)$row[1],   // pocet prihlasenych
//                $row[0] - $row[1]   // pocet volnych mist
//            ];
//        }
//
//        return [null, null, -1];
//    }



    /**
     * @return BaseViewObject|Redirect
     * @throws DBException
     * @throws Exception
     */
    public function vyslatNaSkoleni()
    {
        // pripoji js pro nacteni volne kapacity kurzu
        HtmlHeaders::getInstance()->addScript('script/personscripts.js');

        $this->hlavickaVyberAkce();

        if (!Prava::vratHodnotuPrava('skoleni_vyslat')) {
            throw new NeniPravoException();
        }

        $form = new PozadavekNaSkoleniForm($this ->database);
        if ($form ->handleAction()) {
            return new Redirect(NULL, '?menuid=skoleni&action=vyslatnaskoleni');
        }
        return $form;

//        $osoba = isset($_GET['osoba']) ? (int)$_GET['osoba'] : null;
//        $druhSkoleni = isset($_GET['druhskoleni']) ? $_GET['druhskoleni'] : null;   // StringUtils::fromUTF8($_GET['druhskoleni']) ? ? ?
//        $kurz = isset($_GET['kurz']) ? $_GET['kurz'] : null;
//
//        $comboOsoby = $this->osobaVyslanaNaSkoleni();
//        $combo_typ_skoleni = $this->comboTypSkoleni($this->database, $osoba, $druhSkoleni);
//        $combo_kurz = $this->comboKurz($this->database, $osoba, $druhSkoleni, $kurz);
//
//        $naplnenostKurzu = $this->naplnenostKurzu($kurz);
//        Model::addData('combo_typ_skoleni', $combo_typ_skoleni);
//        Model::addData('combo_kurz', $combo_kurz);
//        Model::addData('comboOsoby', $comboOsoby);
//        Model::addData('kurz', $kurz);
//        Model::addData('naplnenostKurzu', $naplnenostKurzu);
//
//        return new BaseViewObject('person/view_zadaniskolenizamestnanci.php');
    }


//    /**
//     * @throws Exception
//     */
//    public function zapisVyslaniZamestnanceNaSkoleni()
//    {
//        $hodnoty = new stdClass();
//        foreach ($_POST as $key => $value) {
//            switch ($key){
//                case "pozadovanedatumskoleni":
//                    try{
//                        if ($value != "") {
//                            $hodnoty->$key = DateUtils::dateStringToTimestamp($value);
//                        }
//                    } catch (Exception $e) {
//                        MessageManager::addErrorMessage($e, $key);
//                        $hodnoty->$key = null;
//                    }
//                    break;
//
//                default:
//                    $hodnoty->$key = filter_input(INPUT_POST, $key, FILTER_SANITIZE_SPECIAL_CHARS);
//            }
//        }
//
//        if ($hodnoty->osoba == ''){
//            throw new Exception(StringUtils::fromUTF8('Není zadaná osoba!'));
//        }
//        if ($hodnoty->druhSkoleni == ''){
//            throw new Exception(StringUtils::fromUTF8('Není zadán typ školení!'));
//        }
//        if ($hodnoty->pozadovanedatumskoleni == ''){
//            throw new Exception(StringUtils::fromUTF8('Není zadáno požadované datum školení!'));
//        }
//
//        if ($hodnoty->pozadovanedatumskoleni < DateUtils::today()){
//            throw new Exception(StringUtils::fromUTF8('Na Portále není možno vyslat na školení do minulosti!'));
//        }
//
//        if ($hodnoty->volnych_mist == 0 && $_GET['kurz'] != ''){
//            throw new Exception(StringUtils::fromUTF8('Kurz vyčerpán!'));
//        }
//
//        Prava::nastavObdobiProPrava(time(), time());
//        if (!(Prava::lzeZobrazitPP('skoleni_vyslat', $hodnoty->osoba, time(), time()))){
//            throw new NeniPravoException();
//        }
//
//        $pracovniPomer = new PracovniPomer();
//        $pracovniPomer->nactiPPPodleId($this->database, $hodnoty->osoba);
//        $pracovniPomer->vyberNactenyPP($this->database, Prava::vratPrava('skoleni_vyslat'));
//
//        $record = new RonPortalDBRecord($this->database, 'GenerovaneSkoleni', $this);
//
//        $record->addField('Prihlasil', '', FALSE, 25, User::getLoginName());
//        $record->addField('RC', '', FALSE, 15, $pracovniPomer->rc);
//        $record->addField('DruhSkoleni', '', FALSE, 10, $hodnoty->druhSkoleni);
//        $record->addField('CisloSkoleni', '', FALSE, 10, $hodnoty->kurz);
//        $record->addField('PopisSkoleni', '', FALSE, 80, $hodnoty->popis_skoleni);
//        $record->addField('DatumVznikuPovinnosti', 'timestamp', FALSE, 0, DateUtils::today());
//        $record->addField('DuvodSkoleni', 'int', FALSE, 0, 1);
//        $record->addField('Potvrzeni', 'int', FALSE, 0, 1);
//        $record->addField('Generovane', 'int', FALSE, 0, 0);
//        $record->addField('PrihlasenoZAplikace', 'int', FALSE, 0, 1);
//        $record->addField('Poznamka', 'memo', FALSE, 0, $hodnoty->poznamka_skoleni);
//        $record->addField('PozadovaneDatumSkoleni', 'date', FALSE, 0, $hodnoty->pozadovanedatumskoleni);
//
//        $params = $this->database->createParameters();
//        $sqlWhere =
//            ' RC = ' . $params->addParameter($pracovniPomer->rc) .
//            ' AND DruhSkoleni = ' . $params->addParameter($hodnoty->druhSkoleni) .
//            ' AND DatumVznikuPovinnosti = ' . $params->addParameter($this->database->timestampToParam(DateUtils::today()));
//        $record->setWhere($sqlWhere, $params);
//
//        if (!$record->loadRecord()) {
//            $record->insertRecord();
//        } else {
//            throw new Exception(StringUtils::fromUTF8('Chyba! Zápis vyslání zaměstnance na dané školení s dnešním datem povinnosti již existuje!'));
//        }
//
//
//
//        if (is_uploaded_file($_FILES['dokument']['tmp_name'])) {
//            try {
//                $idDokumentu = DokumentyUtils::vlozDokument($this->database, 'dokument', RonPortalSettings::getIntValue('skoleni_typ_dokumentu'),
//                    $_POST['dokument_popis'], 'Dokument vyslani na skoleni');
//
//                $recDokumentOsoby = new RonPortalDBRecord($this->database, 'DokumentyOsoby', $this);
//                $recDokumentOsoby->addField('RC', '', true, 15, $pracovniPomer->rc);
//                $recDokumentOsoby->addField('id', 'int', true, 0, $idDokumentu);
//                $recDokumentOsoby->addField('GSk_RC', '', false, 15, $pracovniPomer->rc);
//                $recDokumentOsoby->addField('GSk_DruhSkoleni', '', false, 10, $hodnoty->druhSkoleni);
//                $recDokumentOsoby->addField('GSk_Datum', 'timestamp', true, 0, DateUtils::today());
//
//                $recDokumentOsoby->insertRecord();
//            } catch (Exception $e) {
//                MessageManager::addErrorMessage($e);
//            }
//        }
//
//        MessageManager::addInfoMessage( ronText('Skoleni_vyslani_ulozeno') );
//        return new Redirect(NULL, '?menuid=skoleni&action=vyslatnaskoleni');
//    }


//    /**
//     * @param JsonOutputObject $output
//     * @throws DBException
//     */
//    function downloadFirmy($output)
//    {
//        $firmy = [];
//        $firma = [];
//
//        $sql = "SELECT Firma.kodFirmy AS Kod, Firma.nazev1 {fn_concat()} ' (' {fn_concat()} COALESCE(Firma.mesto, '') {fn_concat()} ')' AS Popis
//                FROM Firma
//                ORDER BY Firma.nazev1";
//
//        $cursor = $this ->database->getSqlResult($sql);
//        while ($row = $cursor->fetch()) {
//            $firma['kod'] = StringUtils::jsonText($row[0]);
//            $firma['popis'] = StringUtils::jsonText($row[1]);
//
//            $firmy[] = $firma;
//        }
//        $cursor ->close();
//
//        $output ->addData('items', $firmy);
//        $output->compress = 'gzip';
//    }

//    /**
//     * @param $idPP
//     * @param $druh
//     * @param $datum
//     * @return JsonOutputObject
//     * @throws DBException
//     * @throws Exception
//     */
//    private function detailniInformaceSkoleniKurzu($idPP, $druh, $datum)
//    {
//        $params = $this->database->createParameters();
//
//        $sql = "SELECT D.Kod, D.Aktivni, D.DelkaPlatnosti, D.URLexternihoSkoleni, D.Elearning, {fn_CASTWIDEMEMO(D.Poznamka)}, D.uzivatelska1, " .      //0..6
//                " D.uzivatelska2, D.uzivatelska3, D.uzivatelska4, D.uzivatelska5, Skoleni.Popis, " .               //7..11
//                " Skoleni.DatumPocatkuSkoleni, Skoleni.DatumUkonceniSkoleni, Skoleni.MistoSkoleni, Skoleni.DelkaSkoleniDnu, " .     //12..15
//                " Skoleni.DelkaSkoleniHodin, Skoleni.Kapacita, Skoleni.Skolitel, Skoleni.Hodnotici, {fn_CASTWIDEMEMO(Skoleni.HodnoceniSkoleni)}, " .    //16..20
//                " Skoleni.DlouhodobyKurz, {fn_CASTWIDEMEMO(Skoleni.ObsahSkoleni)}, Skoleni.CisloPlanovanehoSkoleni, GS.DatumVznikuPovinnosti, S_DuvodSkoleni.Popis, " .  //21..25
//                " GS.DatumSkoleni, GS.PozadovaneDatumSkoleni, GS.DruhSkoleni, Skoleni.Kredity, " .          //26..29
//                " {fn_CASTWIDEMEMO(GS.PoznamkaSchvalovani)}, GS.DatumPoslednihoSkoleni, GS.Prihlasil, S_StavPotvrzeni.Popis, GS.Povinne, " .    //30..34
//                " {fn_CASTWIDEMEMO(GS.Poznamka)}, GS.PopisSkoleni, KS.Popis, SpecifikaceKurzu.Popis, " .                             //35..38
//                " S_StavSchvalPrihlNaKurz.Popis, Firma.nazev1, Firma.nazev2, S_AnoNe.Popis, Skoleni.CasPocatkuSkoleni, Skoleni.CasUkonceniSkoleni,  " .        //39..44
//                " S_StavSkoleni.Popis, Umisteni.Popis, GS.PozadovanaPlatnostDo, Skoleni.CisloSkoleni, D.Popis, GS.Potvrzeni " .   //45..50
//
//                " FROM  DruhSkoleni AS D
//                           LEFT JOIN Skoleni ON D.Kod = Skoleni.DruhSkoleni
//                            LEFT JOIN GenerovaneSkoleni GS ON D.Kod = GS.DruhSkoleni
//                             LEFT JOIN KategorieSkoleni KS ON KS.Kod = D.Kategorie
//                              LEFT JOIN Firma ON (Skoleni.SkoliciFirma = Firma.kodFirmy)
//                               LEFT JOIN PracovniPomer PP ON (GS.RC = PP.RC)
//                                LEFT JOIN SpecifikaceKurzu ON (SpecifikaceKurzu.Kod = Skoleni.Specifikace)
//                                 LEFT JOIN S_StavSchvalPrihlNaKurz ON (S_StavSchvalPrihlNaKurz.Kod = GS.SchvaleniPrihlasky)
//                                  LEFT JOIN S_DuvodSkoleni ON (S_DuvodSkoleni.kod = GS.DuvodSkoleni)
//                                   LEFT JOIN S_AnoNe ON (D.Elearning = S_AnoNe.Kod)
//                                    LEFT JOIN S_StavSkoleni ON (S_StavSkoleni.Kod = Skoleni.StavSkoleni)
//                                     LEFT JOIN Umisteni ON (Skoleni.MistoSkoleni = Umisteni.Kod)
//                                      LEFT JOIN S_StavPotvrzeni ON  (S_StavPotvrzeni.Kod = GS.Potvrzeni)
//
//                WHERE (GS.CisloSkoleni = Skoleni.CisloSkoleni OR GS.CisloSkoleni IS NULL ) AND GS.DruhSkoleni = " . $params->addParameter($druh) .
//                " AND PP.ID = " . $params->addParameter($idPP) .
//                " AND GS.DatumVznikuPovinnosti = " . $params->addParameter($this->database->timestampToParam($datum));
//
//        MessageManager::addDebugLog($sql);
//
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        $row = $cursor->fetch();
//
//        $hodnoty = [];
//        if ($row) {
//            $platnost = $this->database->stringToTimestamp($row[47]);
//            $platnostText = ($platnost == '2147483647' ? ronText('Skoleni_platnost_neomezene') : "");
//
//            //Skoleni
//            $hodnoty['Skoleni_absolvovane_caption'] = StringUtils::jsonText("Skoleni_absolvovane_caption");
//
//            $hodnoty['Skoleni_typ_skoleni'] = StringUtils::jsonText($row[28]);
//            $hodnoty['Skoleni_popis_typu_skoleni'] = StringUtils::jsonText($row[49]);
//            //absolvovano uspesne   absolvovano neuspesne neucast, prominuto
//            if (($row[50] == 3) || ($row[50] == 4) || ($row[50] >= 7))
//                $hodnoty['Skoleni_datum'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[26]));
//            else
//                $hodnoty['Skoleni_datum'] = '';
//            $hodnoty['Skoleni_duvod'] = StringUtils::jsonText($row[25]);
//            $hodnoty['Skoleni_delka_platnosti'] = StringUtils::jsonText($platnostText);
//            $hodnoty['skoleni_vysledek'] = StringUtils::jsonText($row[33]);
//            $hodnoty['Skoleni_Kredity'] = StringUtils::jsonText($row[29]);
//            $hodnoty['Skoleni_poznamka_skoleni'] = StringUtils::jsonText($row[35]);
//            $hodnoty['Skoleni_elearning'] = StringUtils::jsonText($row[42]);
//            $hodnoty['Skoleni_kategorie'] = StringUtils::jsonText($row[37]);
//            $hodnoty['Skoleni_poznamka_typu_skoleni'] = StringUtils::jsonText($row[5]);
//
//            if ($row[48]) {
//                //Kurz pokud existuje kod pak nacist informace kurzu
//                $hodnoty['Skoleni_kurz_caption'] = StringUtils::jsonText("Skoleni_kurz_caption");
//
//                $hodnoty['Skoleni_ICAL'] = StringUtils::jsonText("?menuid=skoleni&action=ical&cisloskoleni=$row[48]");
//                $hodnoty['Skoleni_stav_kurzu'] = StringUtils::jsonText(ucfirst($row[45]));
//                $hodnoty['Skoleni_CisloSkoleni'] = StringUtils::jsonText($row[48]);
//                $hodnoty['Skoleni_popis_kurzu'] = StringUtils::jsonText($row[11]);
//                $hodnoty['Skoleni_poznamka'] = StringUtils::jsonText($row[20]);
//                $hodnoty['Skoleni_obsah_kurzu'] = StringUtils::jsonText($row[22]);
//                $hodnoty['Skoleni_schvaleni_prihlasky'] = StringUtils::jsonText($row[39]);
//                $hodnoty['datum_pocatku_skoleni'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[12]));
//                $hodnoty['Skoleni_cas_skoleni'] = StringUtils::jsonText(date('G:i', $this->database->stringToTimestamp($row[43]))) . " - " . StringUtils::jsonText(date('G:i', $this->database->stringToTimestamp($row[44])));
//                $hodnoty['datum_ukonceni_skoleni'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[13]));
//                $hodnoty['misto_skoleni'] = StringUtils::jsonText($row[46]);
//                $hodnoty['delka_skoleni_dnu'] = StringUtils::jsonText($row[15]);
//                $hodnoty['delka_skoleni_hodin'] = StringUtils::jsonText($row[16]);
//                $hodnoty['Skoleni_kapacita'] = StringUtils::jsonText($row[17]);
//                $hodnoty['Skoleni_poznamka_schvalovani'] = StringUtils::jsonText($row[30]);
//                $hodnoty['Skoleni_specifikace'] = StringUtils::jsonText($row[38]);
//                $hodnoty['skolitel'] = StringUtils::jsonText($row[18]);
//                $hodnoty['hodnotici'] = StringUtils::jsonText($row[19]);
//                $hodnoty['Skoleni_skolici_firma'] = StringUtils::jsonText($row[40]) . ($row[41] ? " (" . StringUtils::jsonText($row[41]) . ")" : "");
//            }
//
//            $cursor->close();
//            return new JsonOutputObject($hodnoty);
//        }
//
//        $cursor->close();
//
//        return new JsonOutputObject($hodnoty['existujedetail'] = false);
//    }

    /**
     * @param int $osoba
     * @param $cislo
     * @return JsonOutputObject
     * @throws DBException
     * @throws Exception
     */
//    public function nactiKurzyKPrihlaseni($osoba, $cislo)
//    {
//        $json = new JsonOutputObject();
//
//        $seznamKurzu = [];
//        //pravo na osobu jinak ven
//        $pp = new PracovniPomer();
//        $pp->nactiPPPodleId($this->database, $osoba);
//
//        $prava = Prava::vratPrava('skoleni_prihlasit');
//        if (!$prava->lzeZobrazitRC($pp->rc)) {
//            $json->setResult(ronText('Nemate_pravo'));
//            return $json;
//        }
//
//        $params = $this->database->createParameters();
//        //Nabidnout kurzy ktere jsou ve stejnem dni a miste jako zvoleny kurz
//
//        $sql = "SELECT S.CisloSkoleni, S.Popis, S.DruhSkoleni, S.DatumPocatkuSkoleni, S.CasPocatkuSkoleni, S.MistoSkoleni, S.Kapacita,
//					GS.CisloSkoleni, GS.DatumVznikuPovinnosti, GS.Potvrzeni
//                FROM SKOLENI S
//                    INNER JOIN SKOLENI SK ON (( S.MistoSkoleni = SK.MistoSkoleni )
//                        AND (S.DatumPocatkuSkoleni = SK.DatumPocatkuSkoleni) AND (S.CasPocatkuSkoleni = SK.CasPocatkuSkoleni)
//                        AND (SK.CISLOSKOLENI = " . $params->addParameter($cislo) . "))
//                INNER JOIN Umisteni U ON (SK.MistoSkoleni = U.Kod)
//                INNER JOIN DruhSkoleni DS ON (DS.Kod = S.DruhSkoleni)
//                    LEFT OUTER JOIN GenerovaneSkoleni GS ON (GS.RC = " . $params->addParameter($pp->rc) . " AND DS.Kod = GS.DruhSkoleni and GS.Potvrzeni in (0,1) AND (GS.DatumSkoleni IS NULL OR GS.CisloSkoleni = S.CisloSkoleni))
//                WHERE (
//                    EXISTS(SELECT RC FROM GenerovaneSkoleni GSP WHERE GSP.RC = " . $params->addParameter($pp->rc) . " AND GSP.DruhSkoleni = DS.Kod AND (GSP.CisloSkoleni = S.CisloSkoleni OR GSP.DatumSkoleni IS NULL))
//                    OR
//                    EXISTS(SELECT PozadovanaSkoleni.DruhSkoleni
//                        FROM PozadovanaSkoleni
//                        JOIN RozdeleniOsob ON (RozdeleniOsob.KodTypuRozdeleni = PozadovanaSkoleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = PozadovanaSkoleni.KodRozdeleni)
//                        WHERE PozadovanaSkoleni.DruhSkoleni = DS.Kod
//                        AND PozadovanaSkoleni.Povinne = 0
//                        AND RozdeleniOsob.RC = " . $params->addParameter($pp->rc) . "
//                        AND (RozdeleniOsob.PlatnostOd <= " . $params->addParameter($this->database->timestampToParam(time())) . "
//                           AND (RozdeleniOsob.PlatnostDo >= " . $params->addParameter($this->database->timestampToParam(time())) . " OR RozdeleniOsob.PlatnostDo IS NULL))
//                        AND NOT EXISTS(SELECT RC FROM GenerovaneSkoleni GS WHERE GS.RC = RozdeleniOsob.RC AND GS.DruhSkoleni = DS.Kod AND GS.Potvrzeni = 1 AND GS.DatumSkoleni IS NULL)
//                        )
//                    )
//                ORDER BY 1,10,9 DESC";
//
//        MessageManager::addDebugLog($sql);
//        $cursor = $this->database->getSqlResult($sql, $params);
//
//        while ($row = $cursor->fetch()){
//            $hodnoty = [];
//            if ($row[8] != ""){
//                $datumPovinnosti = $this->database->stringToTimestamp($row[8]);
//            } else {
//                $datumPovinnosti = 0;
//            }
//
//            if (!$dalsiKurzNeoznacit){
//                $dalsiKurzNeoznacit = $row[7];
//            }
//
//            //pokud neni jiz prihlasen pak oznaci zvoleny kurz v dialogu, aby jej nemusel opakovane vybirat - vyjma pripadu, kdy je jiz nektery oznaceny
//            $prihlasen = $row[7] == $row[0] ? ronText("Skoleni_Prihlasen") : ronText("Skoleni_Neprihlasen");
//            $oznacitKurz = ($row[7] == $cislo && !$dalsiKurzNeoznacit) ? 1 : 0;
//
//            $hodnoty['Kurz_idpp'] = StringUtils::jsonText($osoba);
//            $hodnoty['Kurz_cislo'] = StringUtils::jsonText($row[0]);
//            $hodnoty['Kurz_druh'] = StringUtils::jsonText($row[2]);
//            $hodnoty['Kurz_oznacit'] = StringUtils::jsonText($oznacitKurz);
//            $hodnoty['Kurz_vznik_povinnosti'] = StringUtils::jsonText($datumPovinnosti);
//            $hodnoty['Kurz_prihlasen_text'] = StringUtils::jsonText($prihlasen);
//		    $hodnoty['Kurz_prihlasen'] = StringUtils::jsonText(($row[7] == $row[0]));
//            $hodnoty['Kurz_Popis'] = "(" . StringUtils::jsonText($row[0]) . ") " . StringUtils::jsonText($row[1]);
//            $hodnoty['Kurz_datum'] = StringUtils::jsonDate($this->database->stringToTimestamp($row[3]));
//
//            $seznamKurzu[] = $hodnoty;
//        }
//        $cursor->close();
//
//
//        $json->addData("kurzy", $seznamKurzu);
//        $json->setResult('OK');
//
//        return $json;
//    }


    /**
     * @throws Exception
     */
    private function hromadnePrihlasitOsobyNaSkoleni()
    {
        $seznamRC = [];

        if(isset($_POST['prihlasitNaSkoleni'])) {
            foreach ($_POST['prihlasitNaSkoleni'] as $item => $radekKurzu) {
                $osoba = $radekKurzu['osoba'];
                $skoleni = $radekKurzu['cislo'];
                $datumPovinnosti = (int)$radekKurzu['datumpovinnosti'];

                if ((isset($radekKurzu['prihlaseno']) && $radekKurzu['prihlaseno']) && (!isset($radekKurzu['prihlasit']))) {
                    //Zrusit skoleni
                    try {
                        $this->odhlaseniSkoleni($skoleni, $osoba, $datumPovinnosti);
                    } catch (DBException $e) {
                        throw $e;
                    } catch (Exception $exception) {
                        MessageManager::addErrorMessage($exception);
                    }
                } elseif ((isset($radekKurzu['prihlaseno']) && $radekKurzu['prihlaseno'] == 0) && (isset($radekKurzu['prihlasit']) && $radekKurzu['prihlasit'] == 1)) {
                    //Prihlasit skoleni
                    try {
                        $rc = $this->prihlaseniSkoleni($skoleni, $osoba, $datumPovinnosti, false);
                        if (!in_array($rc, $seznamRC)) {
                            $seznamRC[] = $rc;
                        }
                    } catch (DBException $e) {
                        throw $e;
                    } catch (Exception $exception) {
                        MessageManager::addErrorMessage($exception);
                    }
                }
            }
        } else {
            MessageManager::addInfoMessage(ronText('Skoleni_hromadne_nevybran_uzivatel'));
        }

        $this->doplnitPracovniPomer($seznamRC);

        return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');
    }


    /**
     * @param string $action
     * @param JsonOutputObject $output
     * @throws Exception
     */
    function handleJsonAction($action, $output)
    {
        switch ($_GET['action']) {
//            case 'seznamfirem' :
//                $this ->downloadFirmy($output);
//                break;

            case 'mojeskoleni' :
                $this ->mojeSkoleni($output);
                break;

            case 'skolenipodrizenych' :
                $this ->skoleniPodrizenych($output);
                break;
        }
    }


    /**
     * @param string $action
     * @return BaseViewObject|InternalRedirect|JsonOutputObject|OutputObject|Redirect|ViewObject|string
     * @throws DBException
     * @throws Exception
     */
    public function handleAction($action)
    {
        if (!RonPortalSettings::getIntValue(RonPortalSettings::ZOBRAZIT_SKOLENI)) {
            throw new NeniPravoException();
        }

        Prava::nastavObdobiProPrava(time(), time());

        $skoleniAction = isset($_GET['action']) ? $_GET['action'] : '';

//        if (isset($_POST['zaznamenatvyslaninaskoleni']) && $_POST['zaznamenatvyslaninaskoleni']){
//            return $this->zapisVyslaniZamestnanceNaSkoleni();
//        }

//        if (isset($_POST['timestampTerminuSkoleni'])  &&  $_POST['timestampTerminuSkoleni']) {
//            return $this ->zapisUcastiNaSkoleni($_POST['timestampTerminuSkoleni']);
//        }

//        if (isset($_POST['hodnoceniSkoleni'])  &&  $_POST['hodnoceniSkoleni']) {
//            return $this ->zapisHodnoceniSkoleni();
//        }

//        if (isset($_POST['vytvoritNovyTerminBtn'])  &&  $_POST['vytvoritNovyTerminBtn'] != "" && ($_POST['pridaniTerminu'] != "")) {
//            return $this ->vytvoreniTerminuSkoleni($_POST['kodKurzu'], $_POST['pridaniTerminu'],
//                $_POST['pridaniTerminu_time'], $_POST['poznamkaKNovemuTerminu']);
//        }

//        if (isset($_POST['timestampEditovanehoTerminuSkoleni'])  &&  $_POST['timestampEditovanehoTerminuSkoleni']) {
//            return $this ->editaceTerminuSkoleni($_POST['kodKurzu'],$_POST['timestampEditovanehoTerminuSkoleni'],
//                $_POST['time'], $_POST['poznamkaKEditovanemuTerminu']);
//        }


//        if (isset($_POST['upravaplanovanehoskoleni'])  &&  $_POST['upravaplanovanehoskoleni'] == 1) {
//            return $this->zapisPlanovanehoSkoleni($_POST['cisloSkoleni']);
//        }
//        if (isset($_POST['nakladyplanovanehoskoleni'])  &&  $_POST['nakladyplanovanehoskoleni']) {
//            return $this->zapisPlanovanehoSkoleni($_POST['cisloSkoleni']);
//        }
//        if (isset($_POST['novenakladyplanovanehoskoleni'])  &&  $_POST['novenakladyplanovanehoskoleni']) {
//            return $this->zapisPlanovanehoSkoleni($_POST['cisloSkoleni']);
//        }

        //Hromadne prihlaseni / odhlaseni osob na skoleni
        if (isset($_POST['hromadnePrihlasitSkoleni']) && $_POST['hromadnePrihlasitSkoleni'] == 1){
            $this->hromadnePrihlasitOsobyNaSkoleni();
        }

//        if ($formaction) {
//            // formular (editace, insert)
//            return $this ->absolvovaniSkoleniFormular($formaction);
//        }


        $redirectAction = null;

        try {
            switch ($skoleniAction) {
                // json akce
                case 'mojeskoleni' :        // dashboard
                case 'skolenipodrizenych' : // dashboard
                    //case 'seznamfirem' :
                    return parent::handleAction($skoleniAction);

                case 'translations':
                    return $this->translations();

                case 'prehled':
                    return $this->prehledSkoleni();

                case 'zobrazprihlaseni':
                    return $this->zobrazPrihlaseniSkoleni();

                case 'vyslatnaskoleni' :
                    return $this->vyslatNaSkoleni();

                case 'planovaneskolenikurzy':
                    //Browser a formular planovanych skoleni
                    return $this->zobrazPlanovaneSkoleni();

                case 'skolenikurzy':
                    return $this->zobrazSkoleniKurzy();

                case 'skoleniucast':    // ucast na dlouhodobem kurzu (proklik ze seznamu skolenikurzy)
                    return $this->zobrazUcastSkoleni();

                case 'skolenihodnoceni':    // odkaz "Hodnoceni ucastniku kurzu" z skolenikurzy ("Lektorem spravovane kurzy")
                    return $this->zobrazHodnoceniUcastniku($_GET['record']);

                case 'zapisabsolvovani':
                    return $this->absolvovaniSkoleniFormular();

                case 'prihlaseninaplanovaneskoleni':
                    $this->hlavickaVyberAkce();
                    $prihlaseniPlanController = new PrihlaseniNaPlanovaneSkoleniController($this->database);
                    return $prihlaseniPlanController->handleAction('');

                case 'prihlaseni':  // odkaz ze seznamu "Prihlaseni na kurz"
                    $redirectAction = 'zobrazprihlaseni';
                    Csrf::validateCSRFGetParam();
                    $this->prihlaseniSkoleni($_GET['cisloskoleni'], $_GET['osoba'], (int)$_GET['datumpovinnosti']);
                    return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');

                case 'odhlaseni':   // odkaz ze seznamu "Prihlaseni na kurz"
                    $redirectAction = 'zobrazprihlaseni';
                    Csrf::validateCSRFGetParam();
                    $this->odhlaseniSkoleni($_GET['cisloskoleni'], $_GET['osoba'], (int)$_GET['datumpovinnosti']);
                    return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');

                case 'schvaleniprihlaseni': // odkaz Schvalit ze seznamu "Prihlaseni na kurz"
                    $redirectAction = 'zobrazprihlaseni';
                    $this->nastavSchvaleniPrihlaseni($_GET['cisloskoleni'], $_GET['osoba'], (int)$_GET['datumpovinnosti']);
                    return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');

                case 'zamitnutiprihlaseni':
                    $redirectAction = 'zobrazprihlaseni';
                    $this->nastavSchvaleniPrihlaseni($_GET['cisloskoleni'], $_GET['osoba'], (int)$_GET['datumpovinnosti'], 2);
                    return new Redirect(null, '?menuid=skoleni&action=zobrazprihlaseni');

                case 'download':
                    return $this->downloadDokumentSkoleni();

                case 'ical' :
                    return $this->downloadIcal();
            }
        } catch (DBException $e) {
            throw $e;
        } catch (Exception $e) {
            if ($redirectAction) {
                MessageManager::addErrorMessage($e);
                return new Redirect(null, '?menuid=skoleni&action=' . $redirectAction);
            }
            throw $e;
        }

        HtmlHeaders::getInstance() ->addCss('css/person.css');
        //HtmlHeaders::getInstance() ->addCss('css/skoleni.css');
        HtmlHeaders::getInstance() ->addCss('person/dotaznik.css'); // PROC?

        HtmlHeaders::getInstance() ->addScript('script/hodnoceni.js');
//        HtmlHeaders::getInstance() ->addScript('script/skolenihodnoceni.js');

        $skoleni = isset($_GET['cisloskoleni']) ? $_GET['cisloskoleni'] : '';

        $redirectAction = null;

        try {
            switch ($skoleniAction) {
                case 'hodnoceni':
                    return $this->hodnoceniSkoleni();

//            case 'nactikurzykprihlaseni':
//                return $this->nactiKurzyKPrihlaseni($_GET['osoba'], $_GET['cislo']);

//                case 'vyslatnaskoleni':
//                    return $this->vyslatNaSkoleni();

//                case 'detailplanovanehoskolenikurzu':
//                    $this->hlavickaVyberAkce();
//                    //View
//                    return $this->detailPlanovaneSkoleni();

//                case 'vytvoreniskolenikurzu':
//                    //View
//                    return $this->vytvoreniPlanovanehoSkoleni(isset($_GET['record']) ? $_GET['record'] : null); // toto je nejaka blbost

//                case 'detailnipopisskolenikurzu':
//                    return $this->detailniInformaceSkoleniKurzu($_GET['idpp'], StringUtils::fromUTF8($_GET['druhskoleni']), $_GET['datum']);

                default:
                    return $this->prehledSkoleni();
            }
        } catch (DBException $e) {
            throw $e;
        } catch (Exception $e) {
            if ($redirectAction) {
                MessageManager::addErrorMessage($e);
                return new Redirect(null, '?menuid=skoleni&action=' . $redirectAction);
            }
            throw $e;
        }
    }
}