<?php
/**
 * stanislav.elbl, 28.06.2022 10:46
 */

namespace Ron\Http;

use Ron\GuiUtils;
use Ron\TextManager;

class Csrf
{
    const CSRF_RANDOM_LEN = 8;


    private static function getCsrfToken()
    {
        // pri prihlaseni vygeneruje nahodny csrf token platny po celou session
        if (!isset($_SESSION['csrf_token'])  ||  !$_SESSION['csrf_token']) {
            $_SESSION['csrf_token'] = bin2hex(random_bytes(16));
        }

        return $_SESSION['csrf_token'];
    }

    /**
     * Vygeneruje CSRF klic pro zadany request. Tento klic pak lze pouzit pro kontrolu metodou validateCSRFKey
     * @return string
     */
    public static function getCsrfKey()
    {
        global $csrf;
        if (!$csrf) {
            return '';
        }
//        global $CSRF_key;
//        if (!$CSRF_key) {
//            return '';
//        }

        $randomStr = bin2hex(random_bytes(self::CSRF_RANDOM_LEN));
        return $randomStr . hash_hmac('sha1', $randomStr, self::getCsrfToken());
        //return sha1($requestString. $CSRF_key . $_SESSION['csrf_token']);
    }


    /**
     * Vygeneruje CSRF klic pro zadany request (viz metoda csrfKeyForRequest) a vraci url parametr s timto klicem
     * @param string $keyParameterName
     * @return string
     */
    public static function csrfUrlParameter($keyParameterName = 'csrfkey')
    {
        $key = self::getCsrfKey();
        if ($key) {
            return '&' . $keyParameterName . '=' . $key;
        }

        return '';
    }


    public static function csrfHiddenInput($htmlParams = [])
    {
        $key = self::getCsrfKey();
        if ($key) {
            return "<input " . GuiUtils::getAttributesHtml(array_merge(
                ['type' => 'hidden', 'name' => 'csrfkey'],
                $htmlParams
            )) . " value='$key'/>";
        }

        return '';
    }

    public static function validateCSRFKey($key)
    {
        if (hash_hmac('sha1', substr($key, 0, 2*self::CSRF_RANDOM_LEN), self::getCsrfToken()) != substr($key, 2*self::CSRF_RANDOM_LEN)) {
            throw new \Exception(TextManager::_('Overovaci_klic_jiz_neni_platny'));
        }
    }

    /**
     * Overi platnost CSRF klice z GET pozadavku pro zadany requestString
     * @param string $keyParameterName
     * @throws \Exception
     */
    public static function validateCSRFGetParam($keyParameterName = 'csrfkey')
    {
        global $csrf;
        if ($csrf) {
            self::validateCSRFKey(isset($_GET[$keyParameterName]) ? $_GET[$keyParameterName] : '');
//            if (!isset($_GET[$keyParameterName])  ||  $_GET[$keyParameterName] !== self::csrfKeyForRequest($requestString)) {
//                throw new \Exception(TextManager::_('Overovaci_klic_jiz_neni_platny'));
//            }
        }
    }


    /**
     * Overi platnost CSRF klice z POST pozadavku pro zadany requestString
     * @param string $keyParameterName
     * @throws \Exception
     */
    public static function validateCSRFPostParam($keyParameterName = 'csrfkey')
    {
        global $csrf;
        if ($csrf) {
            self::validateCSRFKey(isset($_POST[$keyParameterName]) ? $_POST[$keyParameterName] : '');
        }
    }
}
