<?php
/**
 * stanislav.elbl, 26.12.2015
 */

namespace RonPortal\Jidelna;

use Exception;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\DBException;
use Ron\GuiUtils;
use Ron\Http\HttpResponse;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\NastaveniUzivatele;
use RonPortal\Prava;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;


class JidelnicekFilterForm extends RonPortalFilterForm
{
    public $jidelny = [];

    /** @var int */
    public $datumJidelnickuOd;
    /** @var int */
    public $datumJidelnickuDo;

    /** @var array Seznam typu jidel: key = kod typu jidla, value = popis typu jidla */
    private $typyJidel = [];
    /** @var array vybrane typy, jen kody */
    private $vybraneTypy = [];

    function __construct($database, $zobrazitObjednavky)
    {
        parent::__construct($database, 'jidelnicek', true);

        $this ->getAction = $_SERVER['SCRIPT_NAME']; // kvuli odstraneni #..., jinak by nebylo potreba

        if ($zobrazitObjednavky) {
            $prava = Prava::vratPrava('objednani');

            if ($prava->jsouNejakePrava()) {
                $this->nastavOsoby(true, $prava, time(), time(), true); // vcetne externich = true
            }

            $this->nastavJidelnu(true);
        } else {
            $this ->skrytTlacitkoNastavit = true;
        }
    }

    /**
     * @throws DBException
     */
    private function loadTypyJidel()
    {
        $params = $this ->database ->createParameters();

        if ($this ->vybranaOsobaRC) {
            // nacte jen typy jidel povolene v cenove kategorii
            $sql =
                'SELECT DISTINCT TypDennihoJidla.Kod, TypDennihoJidla.Popis, TypDennihoJidla.Poradi' .
                ' FROM TypDennihoJidla, PravidlaCenoveKategorie, RozdeleniOsob, VlastnostCenoveKategorie' .
                " WHERE TypDennihoJidla.Aktivni = 1 AND Kod <> 'ZB'" . // typ Zbozi se nezobrazuje
                // odfiltruje typy jidel, ktere se namaji zobrazovat
                '  AND TypDennihoJidla.Kod = PravidlaCenoveKategorie.TypDennihoJidla' .
                '  AND PravidlaCenoveKategorie.PovolitObjednavky = 1' .
                // jen pravidla platna v zadanem rozsahu dnu ...
                '  AND PravidlaCenoveKategorie.KodTypuRozdeleni = VlastnostCenoveKategorie.KodTypuRozdeleni' .
                '  AND PravidlaCenoveKategorie.CenovaKategorie = VlastnostCenoveKategorie.CenovaKategorie' .
                '  AND PravidlaCenoveKategorie.PlatnostOd = VlastnostCenoveKategorie.PlatnostOd' .
                '   AND VlastnostCenoveKategorie.PlatnostOd <= ' . $params ->addDateParameter($this ->datumJidelnickuDo) .
                '   AND (VlastnostCenoveKategorie.PlatnostDo >= ' . $params ->addDateParameter($this ->datumJidelnickuOd) . ' OR VlastnostCenoveKategorie.PlatnostDo IS NULL) ' .
                // ... prirazena aktualne vybrane osobe
                '   AND VlastnostCenoveKategorie.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
                '   AND VlastnostCenoveKategorie.CenovaKategorie = RozdeleniOsob.Rozdeleni ' .
                '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->vybranaOsobaRC) .
                '   AND RozdeleniOsob.OsobniCislo = ' . $params ->addParameter($this ->vybranaOsobaOsobniCislo) .
                '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($this ->vybranaOsobaDatumNastupu) .
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($this ->datumJidelnickuDo) .
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($this ->datumJidelnickuOd) . ' OR RozdeleniOsob.PlatnostDo IS NULL) ' .
                // seradit podle poradi
                ' ORDER BY TypDennihoJidla.Poradi';
        } else {
            $sql =
                'SELECT Kod, Popis FROM TypDennihoJidla' .
                " WHERE Aktivni = 1 AND Kod <> 'ZB'" . // typ Zbozi se nezobrazuje
                ' ORDER BY Poradi';
        }

        $cursor = $this->database->getSqlResult($sql, $params);

        //nutne pro porovnani pri ukladani uz. nastaveni, v poli musi byt kody
        $vsechnyTypy = [];

        while ($row = $cursor->fetch()) {
            if (isset($_GET['jidelnicek_typ_' . $row[0]])) {
                $this->vybraneTypy[] = $row[0];
            }
            $vsechnyTypy[] = $row[0];

            $this->typyJidel[$row[0]] = $row[1];
        }

        //nastaveni se ulozi jen pri odeslani filtru (je nastavena osoba)
        if (isset($_GET['osoba'])) {
            NastaveniUzivatele::setUserSettingsItem($this->database, 'jidelnicek_vybrane_typy', implode(';', $this->vybraneTypy), implode(';', $vsechnyTypy));
        }
        //jinak se vycte z uzivatelskeho nastaveni
        else {
            $nastaveniTypu = NastaveniUzivatele::getUserSettingsItem($this->database, 'jidelnicek_vybrane_typy');
            if ($nastaveniTypu != '') {
                $this->vybraneTypy = explode(';', $nastaveniTypu);
            }
        }
    }

    /**
     * Zjisti, zda ma zamestnanec nastavenou jednu hlavni jidelnu. Pokud ano, nastavi ji jako vybranou do session.
     * Pokud nema zadnou hlavni nebo ma vice hlavnich, nenastavi nic.
     * POZOR! Uz musi byt nastaveni zamestnanec!
     * @throws Exception
     */
    private function nastavHlavniJidelnuZamestnance()
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT Rozdeleni.kod, Rozdeleni.popis, RozdeleniOsob.Hlavni, RozdeleniOsob.PlatnostOd ' .
            ' FROM Rozdeleni, RozdeleniOsob ' .
            ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
            "   AND RozdeleniOsob.RC = " . $params->addParameter($this ->vybranaOsobaRC) .
            "   AND RozdeleniOsob.OsobniCislo = " . $params->addParameter($this ->vybranaOsobaOsobniCislo) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params->addParameter($this->database->dateToParam( $this ->vybranaOsobaDatumNastupu )) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params->addParameter($this->database->timestampToParam( $this ->datumJidelnickuDo )) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params->addParameter($this->database->timestampToParam( $this ->datumJidelnickuOd )) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND (Rozdeleni.DatumZruseni >= ' . $params->addParameter($this->database->timestampToParam( $this ->datumJidelnickuOd )) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            ' ORDER BY RozdeleniOsob.Hlavni DESC, RozdeleniOsob.PlatnostOd DESC';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $pouzitHlavni = false;
        $hlavniJidelna = null;

        while ($row = $cursor->fetch()) {
            if ((int)$row[2] == 1) {
                if (!$pouzitHlavni) {
                    $pouzitHlavni = true;
                    $hlavniJidelna = $row[0];
                }
                else {
                    $pouzitHlavni = false;
                }
            }
        }

        if ($pouzitHlavni) {
            RonPortalSession::setVybranaJidelna($hlavniJidelna);
        }
    }


    private function nactiVybranouJidelnu()
    {
        //10.7.2017 Po prihlaseni se zjisti, jestli ma zamestnanec nastavenou hlavni jidelnu. Pokud ano, vybere se jako vychozi. Pokud ma vice, nevybere se zadna.
        if (RonPortalSession::vybranaJidelna() == '') {
            $this->nastavHlavniJidelnuZamestnance();
        }

        HttpResponse::setCookieAndSession('jidelna_jidelna', 'vyberjidelny', false, true); // GET

        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Rozdeleni.kod, Rozdeleni.popis ' .
            ' FROM Rozdeleni, RozdeleniOsob ' .
            ' WHERE Rozdeleni.KodTypuRozdeleni = 8 ' .
            '   AND Rozdeleni.KodTypuRozdeleni = RozdeleniOsob.KodTypuRozdeleni ' .
            '   AND Rozdeleni.kod = RozdeleniOsob.Rozdeleni ' .
            '   AND RozdeleniOsob.RC = ' . $params->addParameter($this ->vybranaOsobaRC) .
            '   AND RozdeleniOsob.OsobniCislo = ' . $params->addParameter($this ->vybranaOsobaOsobniCislo) .
            '   AND RozdeleniOsob.DatumNastupu = ' . $params ->addDateParameter($this ->vybranaOsobaDatumNastupu) .
            '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($this ->datumJidelnickuDo) .
            '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($this ->datumJidelnickuOd) . ' OR  RozdeleniOsob.PlatnostDo IS NULL) ' .
            '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($this ->datumJidelnickuOd) . ' OR Rozdeleni.DatumZruseni IS NULL) ' .
            ' ORDER BY Popis';

        $cursor = $this ->database ->getSqlResult($sql, $params);

        $prvniJidelna = null;
        $jeVybranaJidelna = false;

        while ($row = $cursor ->fetch()) {
            if (!isset($prvniJidelna)) {
                $prvniJidelna = $row[0];
            }

            $this ->jidelny[$row[0]] = $row[1];
            if ($row[0] == RonPortalSession::vybranaJidelna()) {
                $jeVybranaJidelna = true;
            }
        }

        if (!$jeVybranaJidelna  &&  isset($prvniJidelna) ) {
            RonPortalSession::setVybranaJidelna($prvniJidelna);
        }
    }


    function loadData()
    {
        $this ->jidelny = [];

        $this ->datumJidelnickuOd = JidelnaController::getDatumJidelnicku();
        $this ->datumJidelnickuDo = DateUtils::incDay($this ->datumJidelnickuOd, 7);

        parent::loadData();

        $this ->nactiVybranouJidelnu();

        $this->loadTypyJidel();
    }


    function renderJidelny()
    {
        echo '<DIV class="filterrow">';
        echo '<SPAN class="filtercaption"></SPAN>';
        echo '<SPAN class="filterformtext">';
        LinksList::renderLinks($this ->jidelny, RonPortalSession::vybranaJidelna(), '?menuid=jidelnicek&vyberjidelny=', '', false);
        echo '</SPAN>';
        echo "</DIV>\n";

    }

    private function renderTypyJidel()
    {
        echo '<div class="filterrow">';
        echo '<SPAN class="filtercaption"></SPAN>';
        echo '<SPAN class="filterformtext">';

        foreach ($this->typyJidel as $kodTypu => $popisTypu) {
            //pokud je uz. nastaveni prazdne jsou vybrane vsechny typy
            GuiUtils::renderCheckbox('jidelnicek_typ_' . $kodTypu, $popisTypu, empty($this->vybraneTypy) ? true : in_array($kodTypu, $this->vybraneTypy), true);
        }

        echo '</SPAN>';
        echo "</DIV>\n";
    }

    public function renderFilters()
    {
        parent::renderFilters();
        $this ->renderJidelny();

        if (RonPortalSettings::getIntValue('jidelnicek_filtrovat_typy', 0)) {
            $this->renderTypyJidel();
        }
    }
}
