<?php
/**
 * Roman.Szablatura 17.1.2017
 */

namespace RonPortal\Terminaly;

use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\MessageManager;
use Ron\TextManager;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;

/**
 * Class ZaznamSnimacuFilterForm
 */
class ZaznamSnimacuFilterForm extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby */
    public $prava;

    //public $rozsahZobrazeni;

    /** @var int    Datum, od ktereho se zobrazi zaznamy */
    public $datumZaznamu;

    /** @var bool   Urcuje, jestli uzivatel omezil rozsah zobrazeneho casu */
    public $volbaCasu = false;
    /** @var string Uzivatelem zadane omezeni casu */
    public $casZaznamuOd = '';
    public $casZaznamuDo = '';

    /** @var int    Pocatecni cas zobrazeni - vypocte se podle datumu a pripadne casu */
    public $timestampOd;
    /** @var int    Konecny cas zobrazeni */
    public $timestampDo;

    /** @var string */
    public $kodUmisteni = null;
    /** @var string */
    public $kodTerminalu = null;
    /** @var string */
    public $kodOperace = null;

    public $zobrazitFiltrUmisteni;

    /** @var string     Urceni zdroje dat (puvodnidochazka/historiepruchodu/all) */
    public $volbaTabulky = 'all';
    /** @var int        Pocet zobrazenych dnu */
    public $pocetDnu = 1;


    public $zobrazitSekundy = true;
    public $zobrazitFotky = false;


    function __construct(Database $database, $action)
    {
        parent::__construct($database, $action, true);

        $this->prava = Prava::vratPrava('prohl_terminaly');
//        $this->f_formaction = '?menuid=zaznamysnimacu';
//        $this->f_classname = 'filtercontainer leftcaptions';

        $this->nastavRozdeleni(true);
        $this->nastavRozdeleni2(true);

        $this->nastavOsoby(true,$this->prava,time(),time(),true);
        $this->povolitVsechyOsoby = true;

        $this->zobrazitFiltrUmisteni = RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitumisteni');
    }


    function renderZaznamyOd()
    {
        //echo '<a name="top"></a>';

        echo '<div class="filterrow" >';
            echo '<span class="filtercaption">';
            echo \Ron\ronText('zobrazit_data_za_pocet_dni');
            echo '</span>';

        echo '<span class="zaznamy_od">';

        $combo = new ComboBox('pocetdnu',[
            1 => TextManager::_('1_den'),
            10 => TextManager::_('10_dni'),
            20 => TextManager::_('20_dni'),
            30 => TextManager::_('30_dni')
        ]);

        $combo->className = 'adsfilterforminput_kod';
        $combo->htmlParams = [
            'data-ron-autosubmit' => '1',
            'title' => \Ron\ronText('zobrazena_doba') . ': ' .
                date('j.n.Y H:i:s', $this ->timestampOd) .
                ' - ' .
                date('j.n.Y H:i:s', $this ->timestampDo)
        ];
        $combo->selectedOption = $combo ->findOption($this ->pocetDnu);
        $combo->render();

        //DateInput pro volbu datumu a pripadne upravu casu
        echo '<span class="padding-left-2 padding-right-2"> ' . \Ron\ronText('Od') . ' </span>';
        DateInput::renderDateSelect('datum', date('j.n.Y', $this->datumZaznamu), true);

        echo '<span title="' . \Ron\ronText('info_cas_bez_upresneni') . '" class="margin-right-2 margin-left-2">';
            GuiUtils::renderCheckbox('volbacasu', \Ron\ronText('upresneni_casu'), $this ->volbaCasu, '', 'clickAutoSubmit');
        echo '</span>';

        $readonlyStr = $this ->volbaCasu ? '' : 'readonly="readonly"';

        echo
            "<input type='text' title='hh:mm' name='casod' placeholder='0:00' value='{$this ->casZaznamuOd}' maxlength='6' size='5' $readonlyStr>" .
            ' - ' .
            "<input type='text' title='hh:mm' name='casdo' placeholder='23:59' value='{$this ->casZaznamuDo}' maxlength='6' size='5' $readonlyStr>";

        echo '</div>';
    }


    /**
     * @throws \Exception
     */
    function renderUmisteni()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            \Ron\ronText('umisteni') .
            '</span>';

        $sql = 'SELECT Kod, Popis FROM Umisteni';

        $combo = new ComboBox('umisteni');
        $combo
            ->setClassName('adsfilterforminput_popis')
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setShowCode(true)
            ->setHtmlParams([
                'id' => 'zaznamy_filtr_umisteni',
                'data-ron-select2' => '#zaznamy_filtr_umisteni_kod',
                'data-ron-autosubmit' => '1'
            ]);

        $combo->loadData($this->database, $sql . ' ORDER BY Popis, Kod', $this ->kodUmisteni);
        $combo->render();

        $combo = new ComboBox(null);
        $combo
            ->setCaptionIndex(0)
            ->setClassName('adsfilterforminput_kod')
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setHtmlParams([
                'id' => 'zaznamy_filtr_umisteni_kod',
                'data-ron-select2' => '#zaznamy_filtr_umisteni',
                'data-ron-autosubmit' => '1'
            ]);

        $combo->loadData($this->database, $sql . " ORDER BY Kod, Popis", $this ->kodUmisteni);
        $combo->render();
        echo '</div>';
    }


    /**
     * @throws \Exception
     */
    function renderTerminal()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            \Ron\ronText('terminal') .
            '</span>';

        $params = $this->database->createParameters();
        if($this ->zobrazitFiltrUmisteni  &&  $this ->kodUmisteni) {
            $sql =
                'SELECT Terminal.Kod, Terminal.Popis' .
                ' FROM Terminal LEFT JOIN Umisteni ON (Terminal.KodUmisteni = Umisteni.Kod)' .
                ' WHERE Terminal.KodUmisteni = ' . $params->addParameter($this ->kodUmisteni);
        } else {
            $sql =
                'SELECT Kod, Popis FROM Terminal';
        }

        $combo = new ComboBox('terminal');
        $combo
            ->setClassName('adsfilterforminput_popis')
            ->setShowCode(true)
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setHtmlParams([
                'id' => 'zaznamy_filtr_terminal',
                'data-ron-autosubmit' => '1',
                'data-ron-select2' => '#zaznamy_filtr_kod_terminalu'
            ]);

        $combo->loadData($this->database,
            $sql . ' ORDER BY Popis, Kod',
            $this ->kodTerminalu, $params);

        $combo->render();

        $combo = new ComboBox(null);
        $combo
            ->setClassName('adsfilterforminput_kod')
            ->setCaptionIndex(0)
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setHtmlParams([
                'id' => 'zaznamy_filtr_kod_terminalu',
                'data-ron-autosubmit' => '1',
                'data-ron-select2' => '#zaznamy_filtr_terminal'
            ]);

        $combo->loadData($this->database,
            $sql . ' ORDER BY Kod',
            $this ->kodTerminalu, $params);
        //$combo->htmlParams = "data-ron-select2='terminal' data-ron-autosubmit='1'";
        $combo->render();

        echo '</div>';
    }

    /**
     * @throws \Exception
     */
    function renderKodOperace()
    {
        echo '<div class="filterrow">';
            echo
                '<span class="filtercaption">' .
                \Ron\ronText('kod_operace') .
                '</span>';

            $sql =
                "SELECT Kod, Popis FROM OperaceDochazky";

            $combo = new ComboBox('kodoperace');
            $combo
                ->setAlwaysShowSelected(true)
                ->setShowCode(true)
                ->setEmptyRow(TextManager::_('_vse_'))
                ->setClassName('adsfilterforminput_popis')
                ->setHtmlParams([
                    'id' => 'zaznamy_filtr_operace',
                    'data-ron-select2' => '#zaznamy_filtr_operace_kod',
                    'data-ron-autosubmit' => '1'
                ]);

            $combo->loadData($this->database, $sql . ' ORDER BY Popis, Kod', $this ->kodOperace);
            $combo->render();

            $combo = new ComboBox(null);
            $combo
                ->setKeyIndex(0) ->setCaptionIndex(0)
                ->setAlwaysShowSelected(true)
                ->setEmptyRow(TextManager::_('_vse_'))
                ->setClassName('adsfilterforminput_kod')
                ->setHtmlParams([
                    'id' => 'zaznamy_filtr_operace_kod',
                    'data-ron-select2' => '#zaznamy_filtr_operace',
                    'data-ron-autosubmit' => '1'
                ]);

            $combo->loadData($this->database, $sql . ' ORDER BY Kod, Popis', $this ->kodOperace);
            $combo->render();
        echo '</div>';
    }


    private function renderDalsiVolby()
    {
        echo '<div id="filterrow_zaznamy_volby" class="filterrow">';
        echo '<span class="filtercaption"></span>';
        echo '<span>';
        GuiUtils::renderCheckbox('zobrazitfotky', \Ron\ronText('foto_v_radku'), $this ->zobrazitFotky, '', 'clickAutoSubmit');

        //Zobrazeni checkboxu sekund
        echo '<span class="margin-left-2">';
        GuiUtils::renderCheckbox('sekundy', \Ron\ronText('zobrazeni_vterin'), $this ->zobrazitSekundy, '', 'clickAutoSubmit');
        echo '</span>';

        echo '</span>';
        echo '</div>';

        echo '<div id="filterrow_zaznamy_zdroj" class="filterrow">';
        echo '<span class="filtercaption"></span>';
        echo '<span>';
        // puvodnidochazka /  historiepruchodu / vse
        LinksList::renderLinks(
            ['puvodnidochazka' => \Ron\ronText('puvodnidochazka'), 'historiepruchodu' => \Ron\ronText('historiepruchodu'), 'all' => \Ron\ronText('Vse')],
            $this ->volbaTabulky, 'volbatabulky','',
            false,
            true); // hidden field
        echo '</span>';
        echo '</div>';
    }


    private function loadVolbaTabulek()
    {
        if (isset($_GET['volbatabulky'])) {
            switch ($_GET['volbatabulky']) {
                case 'puvodnidochazka' :
                case 'historiepruchodu' :
                    $this ->volbaTabulky = $_GET['volbatabulky'];
                    break;

                default :
                    $this ->volbaTabulky = 'all';
            }

            $_SESSION['ZaznamySnimacu_VolbaTabulky'] = $this ->volbaTabulky;
        } else {
            //Pri prvnim zobrazeni stranky se implicitne nastavi all
            $this ->volbaTabulky = $_SESSION['ZaznamySnimacu_VolbaTabulky'] ?? 'all';
        }
    }


    private function loadFotoVeStrance()
    {
        if (isset($_GET['pocetdnu'])) {
            $this ->zobrazitFotky = isset($_GET['zobrazitfotky'])  &&  (int)$_GET['zobrazitfotky'];
            $_SESSION['ZaznamySnimacu_ZobrazitFotky'] = $this ->zobrazitFotky;
        } else {
            $this ->zobrazitSekundy = $_SESSION['ZaznamySnimacu_ZobrazitFotky'] ?? 0;
        }
    }


    /**
     * @throws \Exception
     */
    function loadPocetDnu()
    {
        if (isset($_GET['pocetdnu'])) {
            switch ((int)$_GET['pocetdnu']) {
                case 10 :
                case 20 :
                case 30 :
                    $this ->pocetDnu = (int)$_GET['pocetdnu'];
                    break;

                default :
                    $this ->pocetDnu = 1;
            }

            $_SESSION['ZaznamySnimacu_PocetDnu'] = $this ->pocetDnu;
        } else {
            $this ->pocetDnu = $_SESSION['ZaznamySnimacu_PocetDnu'] ?? 1;
        }

        // Pocatecni datum
        if (isset($_GET['datum'])){
            try {
                $this->datumZaznamu = DateUtils::dateStringToTimestamp($_GET['datum']);
            } catch(\Exception $e) {
                //MessageManager::addErrorText( 'Bad_time_format' );
                MessageManager::addErrorText($e ->getMessage());
                $this->datumZaznamu = DateUtils::today();
            }

            $_SESSION['ZaznamySnimacu_Datum'] = $this ->datumZaznamu;
        } else {
            $this ->datumZaznamu = $_SESSION['ZaznamySnimacu_Datum'] ?? DateUtils::today();
        }


        if (isset($_GET['casod'])  &&  isset($_GET['casdo'])) {
            $this ->volbaCasu = isset($_GET['volbacasu'])  &&  (int)$_GET['volbacasu'];
            $_SESSION['ZaznamySnimacu_VolbaCasu'] = $this ->volbaCasu;

            $this ->casZaznamuOd = $_GET['casod'];
            $this ->casZaznamuDo = $_GET['casdo'];
        } else {
            $this ->volbaCasu = $_SESSION['ZaznamySnimacu_VolbaCasu'] ?? false;
            $this ->casZaznamuOd = $_SESSION['ZaznamySnimacu_CasOd'] ?? '';
            $this ->casZaznamuDo = $_SESSION['ZaznamySnimacu_CasDo'] ?? '';
        }

        try {
            $this ->timestampOd = ($this ->volbaCasu  &&  $this ->casZaznamuOd != '')
                ? DateUtils::timeStringToTimestamp($this ->casZaznamuOd, $this ->datumZaznamu)
                : $this ->datumZaznamu;

            $this ->casZaznamuOd = Date('G:i', $this ->timestampOd);

            $_SESSION['ZaznamySnimacu_CasOd'] = $this ->casZaznamuOd;
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            $this ->timestampOd = $this ->datumZaznamu;
        }

        try {
            $this ->timestampDo = ($this ->volbaCasu && $this ->casZaznamuDo != '')
                ? DateUtils::timeStringToTimestamp($this ->casZaznamuDo, DateUtils::incDay($this ->datumZaznamu, $this ->pocetDnu - 1))
                : DateUtils::incDay($this ->datumZaznamu, $this ->pocetDnu) - 1; // 1 sekunda pred zacatkem dalsiho dne

            $this ->casZaznamuDo = Date('G:i', $this ->timestampDo);

            $_SESSION['ZaznamySnimacu_CasDo'] = $this ->casZaznamuDo;
        } catch (\Exception $e) {
            MessageManager::addErrorMessage($e ->getMessage());
            $this ->timestampDo = DateUtils::incDay($this ->datumZaznamu, $this ->pocetDnu) - 1; // 1 sekunda pred zacatkem dalsiho dne
        }

//        $this->upresneniCasu(); // TODO: predelat
    }

    function loadSekundy()
    {
        if (isset($_GET['pocetdnu'])) {
            $this ->zobrazitSekundy = isset($_GET['sekundy'])  &&  (int)$_GET['sekundy'];
            $_SESSION['ZaznamySnimacu_Sekundy'] = $this ->zobrazitSekundy;
        } else {
            $this ->zobrazitSekundy = $_SESSION['ZaznamySnimacu_Sekundy'] ?? 0;
        }
    }


    function loadUmisteni()
    {
        if (isset($_GET['umisteni'])) {
            $this ->kodUmisteni = $_GET['umisteni'];
            if (trim($this ->kodUmisteni) === '') {
                $this ->kodUmisteni = null;
            }
            $_SESSION['ZaznamySnimacu_Umisteni'] = $this ->kodUmisteni;
        } else {
            $this ->kodUmisteni = $_SESSION['ZaznamySnimacu_Umisteni'] ?? null;
        }
    }


    function loadTerminal()
    {
        if (isset($_GET['terminal'])) {
            $this ->kodTerminalu = $_GET['terminal'];
            if (trim($this ->kodTerminalu) === '') {
                $this ->kodTerminalu = null;
            }
            $_SESSION['ZaznamySnimacu_Terminal'] = $this ->kodTerminalu;
        } else {
            $this ->kodTerminalu = $_SESSION['ZaznamySnimacu_Terminal'] ?? null;
        }
    }

    function loadKodOperace()
    {
        if (isset($_GET['kodoperace'])) {
            $this ->kodOperace = $_GET['kodoperace'];
            if (trim($this ->kodOperace) === '') {
                $this ->kodOperace = null;
            }
            $_SESSION['ZaznamySnimacu_Operace'] = $this ->kodOperace;
        } else {
            $this ->kodOperace = $_SESSION['ZaznamySnimacu_Operace'] ?? null;
        }
    }


    function loadData()
    {
        $this->loadPocetDnu();

        $this->loadFotoVeStrance();
        $this->loadSekundy();
        $this->loadVolbaTabulek();
        $this->loadTerminal();
        $this->loadUmisteni();
        $this->loadKodOperace();

        parent::loadData();
    }

    /**
     * @throws \Exception
     */
    function renderFilters()
    {
        $this->renderZaznamyOd();

        parent::renderFilters();

        if($this->zobrazitFiltrUmisteni){
            $this->renderUmisteni();
        }

        $this->renderTerminal();
        $this->renderKodOperace();
        $this->renderDalsiVolby();
    }
}


/**
 * Class FieldFoto
 * @package Ron\RonPortal\Dochazka
 */
class FieldFoto extends BrowserField
{
    private $browserAction;

    /** @var string */
    private $tabulka;
    /** @var bool */
    private $zobrazitFotky = false;

    public function __construct($database, $action, $tabulka = 'foto', $zobrazitFotky = false)
    {
        parent::__construct($database);

        $this->browserAction = $action;
        $this->tabulka = $tabulka;
        $this->zobrazitFotky = $zobrazitFotky;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($this ->zobrazitFotky) {
            if ($value) {
                return
                    '<div class="browserfoto ronFoto">' .
                    '<img src="?menuid=' . $this->browserAction . '&' . $this->tabulka . '=' . (int)$value . '" alt="' . $this->tabulka . '"/>' .
                    '</div>';
            }
        } else {
            if ($value) {
                /** @noinspection HtmlUnknownTarget */
                return
                    '<span class="ronInfoFoto" data-foto-url="?menuid=' . $this->browserAction . '&' . $this->tabulka . '=' . (int)$value . '"' .
                    ($rowData['col-prijmeni'] ? ' data-foto-alt="' . StringUtils::htmlTextConvert($rowData['col-prijmeni']) . '"' : '') .
                    '>' .
                    '<i class="inline-glyph fa glyph-image"></i>' .
                    //'<img src="images/16_info.png"  alt="' . $this->tabulka . '"/>' .
                    '</span>';
            }
        }

        return '';
    }
}


/**
 * Class FieldDatumCasDen
 * @property string header
 * @property string sortSQLDesc
 * @property int ordering
 * @property string sortSQL
 * @property string align
 * @property string className
 * @package Ron\RonPortal\Dochazka
 */
class FieldDatumCasDen extends BrowserField
{
    var $datumPoslednihoRadku = null;


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $date = $this->database->stringToTimestamp($value);

        $datum = StringUtils::format('date', $date);
        $den = \Ron\ronText('Day_short_' . date("w", $date));

        $result =
            "<span class='padding-right-2' title='$datum'>$den</span>" .
            ($datum != $this ->datumPoslednihoRadku
                ? ' ' . parent::getViewText($value, $rowNumber, $recordId, $rowData)
                : '');

        $this ->datumPoslednihoRadku = $datum;

        return $result;
    }
}



/**
 * Class BrowserZaznamySnimacu
 * @package Ron\RonPortal\Dochazka
 */
class ZaznamySnimacuBrowser extends RonPortalBrowser
{
    /** @var  ZaznamSnimacuFilterForm */
    public $filterForm;

    public $action;

    /** @var PravaNaOsoby  */
    var $pravaProhlizet;


    /**
     * BrowserZaznamySnimacu constructor.
     * @param Database $database
     * @param $action
     * @throws \Exception
     */
    public function __construct(Database $database, $action)
    {
        parent::__construct($database, 'zaznamysnimacu');

        $this->title = \Ron\ronText('zaznamy_snimacu');
        $this->code = 'browserzaznamysnimacu';
        $this->tableId = 'browserzaznamysnimacu';
        $this->browserUrl = '?menuid=zaznamysnimacu&';
        $this->action = $action;

        $this->filterForm = new ZaznamSnimacuFilterForm($this->database, 'zaznamysnimacu');
        $this->filterForm->loadData();

        $this->pravaProhlizet = Prava::vratPrava('prohl_terminaly');

        $this ->maxPageRows = 500;

        $fieldsZaznamy =
            'RC, OsobniCislo, DatumNastupu, KodOperace, Terminal, Karta,' .
            ' VlozenoManualne, VlozenoOnLine,' .
            ' ZemSirka, ZemDelka, PopisPolohy,' .
            ' KodTypuRozdeleni, Rozdeleni, Uzivatel, Poznamka';

        //Volba tabulky pro sqlFromPart
        if ($this->filterForm ->volbaTabulky === 'historiepruchodu') {
            $fieldsZaznamySQL =
                "SELECT DatumCasPruchodu DatumCas, $fieldsZaznamy FROM HistoriePruchodu";
        } else if ($this ->filterForm ->volbaTabulky === 'puvodnidochazka') {
            $fieldsZaznamySQL =
                "SELECT DatumCasOperace DatumCas, $fieldsZaznamy FROM PuvodniDochazka";
        } else {
            $fieldsZaznamySQL =
                "SELECT DatumCasOperace DatumCas, $fieldsZaznamy FROM PuvodniDochazka" .
                ' UNION ALL' .
                " SELECT DatumCasPruchodu DatumCas, $fieldsZaznamy FROM HistoriePruchodu";
        }

        $this->sqlFromPart =
            "($fieldsZaznamySQL) ZaznamySnimacu" .
            "\nLEFT JOIN PracovniPomer ON (PracovniPomer.RC = ZaznamySnimacu.RC AND PracovniPomer.OsobniCislo = ZaznamySnimacu.OsobniCislo AND PracovniPomer.DatumNastupu = ZaznamySnimacu.DatumNastupu)" .
            "\nLEFT JOIN Osoba ON (Osoba.RC = Zaznamysnimacu.RC)" .
            "\nLEFT JOIN OperaceDochazky ON (OperaceDochazky.Kod = ZaznamySnimacu.KodOperace)" .
            "\nLEFT JOIN Terminal ON (Terminal.Kod = ZaznamySnimacu.Terminal)" .
            "\nLEFT JOIN Umisteni ON (Terminal.KodUmisteni = Umisteni.Kod)" .
            "\nLEFT JOIN FotoZTerminalu ON (ZaznamySnimacu.DatumCas = FotoZTerminalu.DatumCasOperace AND ZaznamySnimacu.Terminal = FotoZTerminalu.Terminal)";

        $this->primaryField = 0;
        $this->defaultSort = -1;

        $this->addField(new FieldDatumCasDen($this->database))
            ->setSqlFieldName('ZaznamySnimacu.DatumCas') ->setHeaderText('Datum') ->setFormat('date')
            ->setCode('col-datum') ->setOrdering(true);

        $this ->addField()
            ->setSqlFieldName('ZaznamySnimacu.DatumCas') ->setFormat($this ->filterForm ->zobrazitSekundy ? 'fulltime' : 'time')
            ->setCode('col-cas')
            ->setHeaderText('Cas') ->setOrdering(true);

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.RC')
            ->setCode('col-rc');

        $this->addField(new FieldFoto($this->database, $this->action, 'foto', $this ->filterForm ->zobrazitFotky))
            ->setSqlFieldName('Osoba.IdFotografie')
            ->setCode('col-foto') ->setHeaderText($this ->filterForm ->zobrazitFotky ? 'foto_z_karty' : '');

        $this->addField(new FieldFoto($this->database, $this->action, 'fotozterminalu', $this ->filterForm ->zobrazitFotky))
            ->setSqlFieldName('FotoZTerminalu.ID')
            ->setCode('col-fotozterminalu') ->setHeaderText($this ->filterForm ->zobrazitFotky ? 'foto_z_terminalu' : '');

        // jmeno a prijmeni
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Prijmeni') ->setCode('col-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Jmeno') ->setCode('col-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.Titul') ->setCode('col-titul');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Osoba.TitulZa') ->setCode('col-titulza');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-jmeno', 'col-prijmeni', 'col-titul', 'col-titulza'))
            ->setCode('col-osoba') ->setOrdering(true)
            ->setSortSQL('Osoba.Prijmeni, Osoba.Jmeno, ZaznamySnimacu.DatumCas')
            ->setSortSQLDesc('Osoba.Prijmeni DESC, Osoba.Jmeno DESC, ZaznamySnimacu.DatumCas DESC');

        $this->addField()
            ->setSqlFieldName('PracovniPomer.OsobniCislo') ->setHeaderText('Osobni_cislo')
            ->setCode('col-osobnicislo')
            ->setOrdering(true)
            ->setSortSQL('PracovniPomer.OsobniCislo, ZaznamySnimacu.DatumCas')
            ->setSortSQLDesc('PracovniPomer.OsobniCislo DESC, ZaznamySnimacu.DatumCas DESC');

        $this ->addField()
            ->setSqlFieldName('PracovniPomer.DatumNastupu') ->setHeaderText('Datum_nastupu') ->setFormat('date')
            ->setCode('col-datumnastupu')
            ->setOrdering(true) ->setAlign('left')
            ->setVisible(RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitdatumnastupu'));


        // kod a popis operace
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OperaceDochazky.Kod')
            ->setCode('col-kod-operace');

        $this ->addField()
            ->setSqlFieldName('OperaceDochazky.Popis') ->setHeaderText('kod_operace') ->setOrdering(true)
            ->setCode('col-operace')
            ->setSortSQL('OperaceDochazky.Popis, ZaznamySnimacu.DatumCas')
            ->setSortSQLDesc('OperaceDochazky.Popis DESC, ZaznamySnimacu.DatumCas DESC')
            ->setSqlFieldIndex2('col-kod-operace');

        // kod a popis terminalu
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Terminal.Kod')
            ->setCode('col-kod-terminalu');

        $this ->addField()
            ->setSqlFieldName('Terminal.Popis') ->setHeaderText('terminal') ->setOrdering(true)
            ->setCode('col-terminal')
            ->setSqlFieldIndex2('col-kod-terminalu')
            ->setSortSQL('Terminal.Popis, ZaznamySnimacu.DatumCas')
            ->setSortSQLDesc('Terminal.Popis DESC, ZaznamySnimacu.DatumCas DESC');


        // karta
        $this->addField()
            ->setSqlFieldName('ZaznamySnimacu.Karta') ->setHeaderText('Karta')
            ->setCode('col-karta')
            ->setOrdering(true);

        if ((int)RonPortalSettings::getIntValue('zaznamysnimacu_zobrazitzpusobvlozeni')) {
//            $this->addField([BROWSER_FIELD_SQL_INDEX => 19, BROWSER_FIELD_SQL_NAME => 'S_AnoNeVlozenoManualne.Popis', BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('vlozeno_manualne'),
//                BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center']);

            $this->addField(BrowserField::FIELD_TYPE_SELECT)
                ->setSqlFieldName('VlozenoManualne') ->setHeaderText('vlozeno_manualne') ->setAlign('center') ->setOrdering(true)
                ->setCode('col-vlozenomanualne')
                ->setOptions([0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')]);

//            $this->addField([BROWSER_FIELD_SQL_INDEX => 20, BROWSER_FIELD_SQL_NAME => 'S_AnoNeVlozenoOnLine.Popis', BROWSER_FIELD_HEADER_TEXT => \Ron\ronText('vlozeno_online'),
//                BROWSER_FIELD_ORDERING => 1, BROWSER_FIELD_ALIGN => 'center']);

            $this->addField(BrowserField::FIELD_TYPE_SELECT)
                ->setSqlFieldName('VlozenoOnline') ->setHeaderText('vlozeno_online') ->setAlign('center') ->setOrdering(true)
                ->setCode('col-vlozenoonline')
                ->setOptions([0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')]);
        }

        $this->addField()
            ->setSqlFieldName('ZaznamySnimacu.Uzivatel') ->setHeaderText('Uzivatel') ->setOrdering(true)
            ->setCode('col-uzivatel');

        $this->addField()
            ->setSqlFieldName('ZaznamySnimacu.Poznamka') ->setHeaderText('Poznamka') ->setOrdering(true)
            ->setCode('col-poznamka')
            ->setClassName('poznamka');

        $this ->addField(BrowserField::FIELD_TYPE_LINE)
            ->setCode('col-line-number');  // ->setLink("#top")???
    }


    function setFilterForm()
    {
        // nedela nic - jiz nastaveno v konstruktoru
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        // Zobrazi pokud ma na osobu pravo, nebo neni osoba prirazena
        return !isset($values['col-rc'])  ||  $this->pravaProhlizet->lzeZobrazitRC($values['col-rc']);
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();

        $result =
            'Zaznamysnimacu.DatumCas >= ' . $params->addTimestampParameter($this->filterForm ->timestampOd) .
            ' AND Zaznamysnimacu.DatumCas <= ' . $params->addTimestampParameter($this ->filterForm ->timestampDo);

        if (!$this ->filterForm ->vsechnyOsoby) {
            $result .=
                ' AND Zaznamysnimacu.RC = ' . $params->addParameter($this ->filterForm ->vybranaOsobaRC);
        } else {
            if ($this ->filterForm ->kodRozdeleni != '') {
                $result .=
                    ($result ? ' AND ' : '') .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'ZaznamySnimacu',
                        $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
                        time(), time(),
                        $this ->filterForm ->vcetnePodrizenychRozdeleni,
                        $params);
//                    RozdeleniUtils::sqlFilterRozdeleni($this->database, 'ZaznamySnimacu',
//                        $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
//                        $this ->filterForm ->vcetnePodrizenychRozdeleni,
//                        $params);
            }

            if ($this ->filterForm ->kodRozdeleni2 != '') {
                $result .=
                    ($result? ' AND ' : '') .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'Zaznamysnimacu',
                        $this ->filterForm ->kodTypuRozdeleni2, $this ->filterForm ->kodRozdeleni2,
                        time(), time(),
                        $this ->filterForm ->vcetnePodrizenychRozdeleni2,
                        $params);
            }
        }


        if ($this ->filterForm ->zobrazitFiltrUmisteni  &&  $this ->filterForm ->kodUmisteni) {
            $result .=
                ($result? ' AND ' : '') .
                " Umisteni.Kod = " . $params->addParameter($this ->filterForm ->kodUmisteni);
        }

        if (isset($this ->filterForm ->kodTerminalu)) {
            $result .=
                ($result ? ' AND ' : '') .
                'Zaznamysnimacu.Terminal = ' . $params->addParameter($this ->filterForm ->kodTerminalu);
        }

        if (isset($this ->filterForm ->kodOperace)) {
            $result .=
                ($result ? ' AND ' : '') .
                'Zaznamysnimacu.KodOperace = ' . $params->addParameter($this ->filterForm ->kodOperace);
        }

        return new SqlStatement($result, $params);
    }

    public function render()
    {
        $aktualizace = RonPortalSettings::getValue('zaznamysnimacu_dobaaktualizacestranky', 60);

        if ($aktualizace) {
            echo "<div class='auto-refresh' data-timeout='$aktualizace' data-url='?menuid=zaznamysnimacu'>";
        }

        parent::render();

        if ($aktualizace) {
            echo '</div>';
        }
    }

}